@@ -2,77 +2,80 @@
 
 #include "zeek-config.h"
 
-#include "CompHash.h"
-#include "Val.h"
-#include "Reporter.h"
-#include "Func.h"
-
+#include <cstring>
 #include <vector>
 #include <map>
 
-CompositeHash::CompositeHash(TypeList* composite_type)
+#include "CompHash.h"
+#include "ZeekString.h"
+#include "Dict.h"
+#include "Val.h"
+#include "RE.h"
+#include "Reporter.h"
+#include "Func.h"
+#include "IPAddr.h"
+
+CompositeHash::CompositeHash(zeek::TypeListPtr composite_type)
+	: type(std::move(composite_type))
 	{
-	type = composite_type;
-	Ref(type);
-	singleton_tag = TYPE_INTERNAL_ERROR;
+	singleton_tag = zeek::TYPE_INTERNAL_ERROR;
 
 	// If the only element is a record, don't treat it as a
 	// singleton, since it needs to be evaluated specially.
 
-	if ( type->Types()->length() == 1 )
+	if ( type->GetTypes().size() == 1 )
 		{
-		if ( (*type->Types())[0]->Tag() == TYPE_RECORD )
+		if ( type->GetTypes()[0]->Tag() == zeek::TYPE_RECORD )
 			{
-			is_complex_type = 1;
-			is_singleton = 0;
+			is_complex_type = true;
+			is_singleton = false;
 			}
 		else
 			{
-			is_complex_type = 0;
-			is_singleton = 1;
+			is_complex_type = false;
+			is_singleton = true;
 			}
 		}
 
 	else
 		{
-		is_singleton = 0;
-		is_complex_type = 0;
+		is_singleton = false;
+		is_complex_type = false;
 		}
 
 	if ( is_singleton )
 		{
 		// Don't do any further key computations - we'll do them
 		// via the singleton later.
-		singleton_tag = (*type->Types())[0]->InternalType();
+		singleton_tag = type->GetTypes()[0]->InternalType();
 		size = 0;
-		key = 0;
+		key = nullptr;
 		}
 
 	else
 		{
-		size = ComputeKeySize(0, 1, true);
+		size = ComputeKeySize(nullptr, true, true);
 
 		if ( size > 0 )
 			// Fixed size.  Make sure what we get is fully aligned.
 			key = reinterpret_cast<char*>
 				(new double[size/sizeof(double) + 1]);
 		else
-			key = 0;
+			key = nullptr;
 		}
 	}
 
 CompositeHash::~CompositeHash()
 	{
-	Unref(type);
 	delete [] key;
 	}
 
 // Computes the piece of the hash for Val*, returning the new kp.
-char* CompositeHash::SingleValHash(int type_check, char* kp0,
-				   BroType* bt, Val* v, bool optional) const
+char* CompositeHash::SingleValHash(bool type_check, char* kp0,
+                                   zeek::Type* bt, zeek::Val* v, bool optional) const
 	{
-	char* kp1 = 0;
-	InternalTypeTag t = bt->InternalType();
+	char* kp1 = nullptr;
+	zeek::InternalTypeTag t = bt->InternalType();
 
 	if ( optional )
 		{
@@ -87,13 +90,13 @@ char* CompositeHash::SingleValHash(int type_check, char* kp0,
 
 	if ( type_check )
 		{
-		InternalTypeTag vt = v->Type()->InternalType();
+		zeek::InternalTypeTag vt = v->GetType()->InternalType();
 		if ( vt != t )
-			return 0;
+			return nullptr;
 		}
 
 	switch ( t ) {
-	case TYPE_INTERNAL_INT:
+	case zeek::TYPE_INTERNAL_INT:
 		{
 		bro_int_t* kp = AlignAndPadType<bro_int_t>(kp0);
 		*kp = v->ForceAsInt();
@@ -101,7 +104,7 @@ char* CompositeHash::SingleValHash(int type_check, char* kp0,
 		}
 		break;
 
-	case TYPE_INTERNAL_UNSIGNED:
+	case zeek::TYPE_INTERNAL_UNSIGNED:
 		{
 		bro_uint_t* kp = AlignAndPadType<bro_uint_t>(kp0);
 		*kp = v->ForceAsUInt();
@@ -109,7 +112,7 @@ char* CompositeHash::SingleValHash(int type_check, char* kp0,
 		}
 		break;
 
-	case TYPE_INTERNAL_ADDR:
+	case zeek::TYPE_INTERNAL_ADDR:
 		{
 		uint32_t* kp = AlignAndPadType<uint32_t>(kp0);
 		v->AsAddr().CopyIPv6(kp);
@@ -117,7 +120,7 @@ char* CompositeHash::SingleValHash(int type_check, char* kp0,
 		}
 		break;
 
-	case TYPE_INTERNAL_SUBNET:
+	case zeek::TYPE_INTERNAL_SUBNET:
 		{
 		uint32_t* kp = AlignAndPadType<uint32_t>(kp0);
 		v->AsSubNet().Prefix().CopyIPv6(kp);
@@ -126,7 +129,7 @@ char* CompositeHash::SingleValHash(int type_check, char* kp0,
 		}
 		break;
 
-	case TYPE_INTERNAL_DOUBLE:
+	case zeek::TYPE_INTERNAL_DOUBLE:
 		{
 		double* kp = AlignAndPadType<double>(kp0);
 		*kp = v->InternalDouble();
@@ -134,11 +137,11 @@ char* CompositeHash::SingleValHash(int type_check, char* kp0,
 		}
 		break;
 
-	case TYPE_INTERNAL_VOID:
-	case TYPE_INTERNAL_OTHER:
+	case zeek::TYPE_INTERNAL_VOID:
+	case zeek::TYPE_INTERNAL_OTHER:
 		{
-		switch ( v->Type()->Tag() ) {
-		case TYPE_FUNC:
+		switch ( v->GetType()->Tag() ) {
+		case zeek::TYPE_FUNC:
 			{
 			uint32_t* kp = AlignAndPadType<uint32_t>(kp0);
 			*kp = v->AsFunc()->GetUniqueFuncID();
@@ -146,17 +149,17 @@ char* CompositeHash::SingleValHash(int type_check, char* kp0,
 			break;
 			}
 
-		case TYPE_PATTERN:
+		case zeek::TYPE_PATTERN:
 			{
 			const char* texts[2] = {
 				v->AsPattern()->PatternText(),
 				v->AsPattern()->AnywherePatternText()
 			};
 
-			size_t* kp;
+			uint64_t* kp;
 			for ( int i = 0; i < 2; i++ )
 				{
-				kp = AlignAndPadType<size_t>(kp0+i);
+				kp = AlignAndPadType<uint64_t>(kp0+i);
 				*kp = strlen(texts[i]) + 1;
 				}
 
@@ -170,43 +173,43 @@ char* CompositeHash::SingleValHash(int type_check, char* kp0,
 			break;
 			}
 
-		case TYPE_RECORD:
+		case zeek::TYPE_RECORD:
 			{
 			char* kp = kp0;
-			RecordVal* rv = v->AsRecordVal();
-			RecordType* rt = bt->AsRecordType();
+			zeek::RecordVal* rv = v->AsRecordVal();
+			zeek::RecordType* rt = bt->AsRecordType();
 			int num_fields = rt->NumFields();
 
 			for ( int i = 0; i < num_fields; ++i )
 				{
-				Val* rv_i = rv->Lookup(i);
+				auto rv_i = rv->GetField(i).get();
 
-				Attributes* a = rt->FieldDecl(i)->attrs;
-				bool optional = (a && a->FindAttr(ATTR_OPTIONAL));
+				zeek::detail::Attributes* a = rt->FieldDecl(i)->attrs.get();
+				bool optional = (a && a->Find(zeek::detail::ATTR_OPTIONAL));
 
 				if ( ! (rv_i || optional) )
-					return 0;
+					return nullptr;
 
 				if ( ! (kp = SingleValHash(type_check, kp,
-							   rt->FieldType(i),
+							   rt->GetFieldType(i).get(),
 							   rv_i, optional)) )
-					return 0;
+					return nullptr;
 				}
 
 			kp1 = kp;
 			break;
 			}
 
-		case TYPE_TABLE:
+		case zeek::TYPE_TABLE:
 			{
 			int* kp = AlignAndPadType<int>(kp0);
-			TableVal* tv = v->AsTableVal();
+			zeek::TableVal* tv = v->AsTableVal();
 			*kp = tv->Size();
 			kp1 = reinterpret_cast<char*>(kp+1);
 
 			auto tbl = tv->AsTable();
 			auto it = tbl->InitForIteration();
-			ListVal* lv = new ListVal(TYPE_ANY);
+			auto lv = zeek::make_intrusive<zeek::ListVal>(zeek::TYPE_ANY);
 
 			struct HashKeyComparer {
 				bool operator()(const HashKey* a, const HashKey* b) const
@@ -228,7 +231,7 @@ char* CompositeHash::SingleValHash(int type_check, char* kp0,
 			while ( tbl->NextEntry(k, it) )
 				{
 				hashkeys[k] = idx++;
-				lv->Append(tv->RecoverIndex(k));
+				lv->Append(tv->RecreateIndex(*k));
 				}
 
 			for ( auto& kv : hashkeys )
@@ -237,41 +240,35 @@ char* CompositeHash::SingleValHash(int type_check, char* kp0,
 			for ( auto& kv : hashkeys )
 				{
 				auto idx = kv.second;
-				Val* key = lv->Index(idx);
+				const auto& key = lv->Idx(idx);
 
-				if ( ! (kp1 = SingleValHash(type_check, kp1, key->Type(), key,
-				                            false)) )
-					{
-					Unref(lv);
-					return 0;
-					}
+				if ( ! (kp1 = SingleValHash(type_check, kp1, key->GetType().get(),
+				                            key.get(), false)) )
+					return nullptr;
 
-				if ( ! v->Type()->IsSet() )
+				if ( ! v->GetType()->IsSet() )
 					{
-					Val* val = tv->Lookup(key);
-					if ( ! (kp1 = SingleValHash(type_check, kp1, val->Type(),
-								    val, false)) )
-						{
-						Unref(lv);
-						return 0;
-						}
+					auto val = tv->FindOrDefault(key);
+
+					if ( ! (kp1 = SingleValHash(type_check, kp1, val->GetType().get(),
+								    val.get(), false)) )
+						return nullptr;
 					}
 				}
 
-			Unref(lv);
 			}
 			break;
 
-		case TYPE_VECTOR:
+		case zeek::TYPE_VECTOR:
 			{
 			unsigned int* kp = AlignAndPadType<unsigned int>(kp0);
-			VectorVal* vv = v->AsVectorVal();
-			VectorType* vt = v->Type()->AsVectorType();
+			zeek::VectorVal* vv = v->AsVectorVal();
+			zeek::VectorType* vt = v->GetType()->AsVectorType();
 			*kp = vv->Size();
 			kp1 = reinterpret_cast<char*>(kp+1);
 			for ( unsigned int i = 0; i < vv->Size(); ++i )
 				{
-				Val* val = vv->Lookup(i);
+				const auto& val = vv->At(i);
 				unsigned int* kp = AlignAndPadType<unsigned int>(kp1);
 				*kp = i;
 				kp1 = reinterpret_cast<char*>(kp+1);
@@ -282,25 +279,26 @@ char* CompositeHash::SingleValHash(int type_check, char* kp0,
 				if ( val )
 					{
 					if ( ! (kp1 = SingleValHash(type_check, kp1,
-					                            vt->YieldType(), val, false)) )
-						return 0;
+					                            vt->Yield().get(), val.get(),
+					                            false)) )
+						return nullptr;
 					}
 				}
 			}
 			break;
 
-		case TYPE_LIST:
+		case zeek::TYPE_LIST:
 			{
 			int* kp = AlignAndPadType<int>(kp0);
-			ListVal* lv = v->AsListVal();
+			zeek::ListVal* lv = v->AsListVal();
 			*kp = lv->Length();
 			kp1 = reinterpret_cast<char*>(kp+1);
 			for ( int i = 0; i < lv->Length(); ++i )
 				{
-				Val* v = lv->Index(i);
-				if ( ! (kp1 = SingleValHash(type_check, kp1, v->Type(), v,
+				zeek::Val* v = lv->Idx(i).get();
+				if ( ! (kp1 = SingleValHash(type_check, kp1, v->GetType().get(), v,
 				                            false)) )
-					return 0;
+					return nullptr;
 				}
 			}
 			break;
@@ -308,18 +306,18 @@ char* CompositeHash::SingleValHash(int type_check, char* kp0,
 		default:
 			{
 			reporter->InternalError("bad index type in CompositeHash::SingleValHash");
-			return 0;
+			return nullptr;
 			}
 		}
 
-		break; // case TYPE_INTERNAL_VOID/OTHER
+		break; // case zeek::TYPE_INTERNAL_VOID/OTHER
 		}
 
-	case TYPE_INTERNAL_STRING:
+	case zeek::TYPE_INTERNAL_STRING:
 		{
 		// Align to int for the length field.
 		int* kp = AlignAndPadType<int>(kp0);
-		const BroString* sval = v->AsString();
+		const zeek::String* sval = v->AsString();
 
 		*kp = sval->Len();	// so we can recover the value
 
@@ -330,34 +328,31 @@ char* CompositeHash::SingleValHash(int type_check, char* kp0,
 		}
 		break;
 
-	case TYPE_INTERNAL_ERROR:
-		return 0;
+	case zeek::TYPE_INTERNAL_ERROR:
+		return nullptr;
 	}
 
 	return kp1;
 	}
 
 
-HashKey* CompositeHash::ComputeHash(const Val* v, int type_check) const
+std::unique_ptr<HashKey> CompositeHash::MakeHashKey(const zeek::Val& argv, bool type_check) const
 	{
-	if ( ! v )
-		reporter->InternalError("null value given to CompositeHash::ComputeHash");
+	auto v = &argv;
 
 	if ( is_singleton )
 		return ComputeSingletonHash(v, type_check);
 
-	if ( is_complex_type && v->Type()->Tag() != TYPE_LIST )
+	if ( is_complex_type && v->GetType()->Tag() != zeek::TYPE_LIST )
 		{
-		ListVal lv(TYPE_ANY);
+		zeek::ListVal lv(zeek::TYPE_ANY);
 
 		// Cast away const to use ListVal - but since we
 		// re-introduce const on the recursive call, it should
 		// be okay; the only thing is that the ListVal unref's it.
-		Val* ncv = (Val*) v;
-		ncv->Ref();
-		lv.Append(ncv);
-	        HashKey* hk = ComputeHash(&lv, type_check);
-		return hk;
+		zeek::Val* ncv = (zeek::Val*) v;
+		lv.Append({zeek::NewRef{}, ncv});
+		return MakeHashKey(lv, type_check);
 		}
 
 	char* k = key;
@@ -366,66 +361,68 @@ HashKey* CompositeHash::ComputeHash(const Val* v, int type_check) const
 		{
 		int sz = ComputeKeySize(v, type_check, false);
 		if ( sz == 0 )
-			return 0;
+			return nullptr;
 
 		k = reinterpret_cast<char*>(new double[sz/sizeof(double) + 1]);
-		type_check = 0;	// no need to type-check again.
+		type_check = false;	// no need to type-check again.
 		}
 
-	const type_list* tl = type->Types();
+	const auto& tl = type->GetTypes();
 
-	if ( type_check && v->Type()->Tag() != TYPE_LIST )
-		return 0;
+	if ( type_check && v->GetType()->Tag() != zeek::TYPE_LIST )
+		return nullptr;
 
-	const val_list* vl = v->AsListVal()->Vals();
-	if ( type_check && vl->length() != tl->length() )
-		return 0;
+	auto lv = v->AsListVal();
+
+	if ( type_check && lv->Length() != static_cast<int>(tl.size()) )
+		return nullptr;
 
 	char* kp = k;
-	loop_over_list(*tl, i)
+	for ( auto i = 0u; i < tl.size(); ++i )
 		{
-		kp = SingleValHash(type_check, kp, (*tl)[i], (*vl)[i], false);
+		kp = SingleValHash(type_check, kp, tl[i].get(), lv->Idx(i).get(), false);
 		if ( ! kp )
-			return 0;
+			return nullptr;
 		}
 
-	return new HashKey((k == key), (void*) k, kp - k);
+	return std::make_unique<HashKey>((k == key), (void*) k, kp - k);
 	}
 
-HashKey* CompositeHash::ComputeSingletonHash(const Val* v, int type_check) const
+std::unique_ptr<HashKey> CompositeHash::ComputeSingletonHash(const zeek::Val* v, bool type_check) const
 	{
-	if ( v->Type()->Tag() == TYPE_LIST )
+	if ( v->GetType()->Tag() == zeek::TYPE_LIST )
 		{
-		const val_list* vl = v->AsListVal()->Vals();
-		if ( type_check && vl->length() != 1 )
-			return 0;
+		auto lv = v->AsListVal();
 
-		v = (*vl)[0];
+		if ( type_check && lv->Length() != 1 )
+			return nullptr;
+
+		v = lv->Idx(0).get();
 		}
 
-	if ( type_check && v->Type()->InternalType() != singleton_tag )
-		return 0;
+	if ( type_check && v->GetType()->InternalType() != singleton_tag )
+		return nullptr;
 
 	switch ( singleton_tag ) {
-	case TYPE_INTERNAL_INT:
-	case TYPE_INTERNAL_UNSIGNED:
-		return new HashKey(v->ForceAsInt());
+	case zeek::TYPE_INTERNAL_INT:
+	case zeek::TYPE_INTERNAL_UNSIGNED:
+		return std::make_unique<HashKey>(v->ForceAsInt());
 
-	case TYPE_INTERNAL_ADDR:
-		return v->AsAddr().GetHashKey();
+	case zeek::TYPE_INTERNAL_ADDR:
+		return v->AsAddr().MakeHashKey();
 
-	case TYPE_INTERNAL_SUBNET:
-		return v->AsSubNet().GetHashKey();
+	case zeek::TYPE_INTERNAL_SUBNET:
+		return v->AsSubNet().MakeHashKey();
 
-	case TYPE_INTERNAL_DOUBLE:
-		return new HashKey(v->InternalDouble());
+	case zeek::TYPE_INTERNAL_DOUBLE:
+		return std::make_unique<HashKey>(v->InternalDouble());
 
-	case TYPE_INTERNAL_VOID:
-	case TYPE_INTERNAL_OTHER:
-		if ( v->Type()->Tag() == TYPE_FUNC )
-			return new HashKey(v->AsFunc()->GetUniqueFuncID());
+	case zeek::TYPE_INTERNAL_VOID:
+	case zeek::TYPE_INTERNAL_OTHER:
+		if ( v->GetType()->Tag() == zeek::TYPE_FUNC )
+			return std::make_unique<HashKey>(v->AsFunc()->GetUniqueFuncID());
 
-		if ( v->Type()->Tag() == TYPE_PATTERN )
+		if ( v->GetType()->Tag() == zeek::TYPE_PATTERN )
 			{
 			const char* texts[2] = {
 				v->AsPattern()->PatternText(),
@@ -435,94 +432,94 @@ HashKey* CompositeHash::ComputeSingletonHash(const Val* v, int type_check) const
 			char* key = new char[n];
 			std::memcpy(key, texts[0], strlen(texts[0]) + 1);
 			std::memcpy(key + strlen(texts[0]) + 1, texts[1], strlen(texts[1]) + 1);
-			return new HashKey(false, key, n);
+			return std::make_unique<HashKey>(false, key, n);
 			}
 
 		reporter->InternalError("bad index type in CompositeHash::ComputeSingletonHash");
-		return 0;
+		return nullptr;
 
-	case TYPE_INTERNAL_STRING:
-		return new HashKey(v->AsString());
+	case zeek::TYPE_INTERNAL_STRING:
+		return std::make_unique<HashKey>(v->AsString());
 
-	case TYPE_INTERNAL_ERROR:
-		return 0;
+	case zeek::TYPE_INTERNAL_ERROR:
+		return nullptr;
 
 	default:
 		reporter->InternalError("bad internal type in CompositeHash::ComputeSingletonHash");
-		return 0;
+		return nullptr;
 	}
 	}
 
-int CompositeHash::SingleTypeKeySize(BroType* bt, const Val* v,
-				     int type_check, int sz, bool optional,
-				     bool calc_static_size) const
+int CompositeHash::SingleTypeKeySize(zeek::Type* bt, const zeek::Val* v,
+                                     bool type_check, int sz, bool optional,
+                                     bool calc_static_size) const
 	{
-	InternalTypeTag t = bt->InternalType();
+	zeek::InternalTypeTag t = bt->InternalType();
 
 	if ( optional )
 		sz = SizeAlign(sz, sizeof(char));
 
 	if ( type_check && v )
 		{
-		InternalTypeTag vt = v->Type()->InternalType();
+		zeek::InternalTypeTag vt = v->GetType()->InternalType();
 		if ( vt != t )
 			return 0;
 		}
 
 	switch ( t ) {
-	case TYPE_INTERNAL_INT:
-	case TYPE_INTERNAL_UNSIGNED:
+	case zeek::TYPE_INTERNAL_INT:
+	case zeek::TYPE_INTERNAL_UNSIGNED:
 		sz = SizeAlign(sz, sizeof(bro_int_t));
 		break;
 
-	case TYPE_INTERNAL_ADDR:
+	case zeek::TYPE_INTERNAL_ADDR:
 		sz = SizeAlign(sz, sizeof(uint32_t));
 		sz += sizeof(uint32_t) * 3;	// to make a total of 4 words
 		break;
 
-	case TYPE_INTERNAL_SUBNET:
+	case zeek::TYPE_INTERNAL_SUBNET:
 		sz = SizeAlign(sz, sizeof(uint32_t));
 		sz += sizeof(uint32_t) * 4;	// to make a total of 5 words
 		break;
 
-	case TYPE_INTERNAL_DOUBLE:
+	case zeek::TYPE_INTERNAL_DOUBLE:
 		sz = SizeAlign(sz, sizeof(double));
 		break;
 
-	case TYPE_INTERNAL_VOID:
-	case TYPE_INTERNAL_OTHER:
+	case zeek::TYPE_INTERNAL_VOID:
+	case zeek::TYPE_INTERNAL_OTHER:
 		{
 		switch ( bt->Tag() ) {
-		case TYPE_FUNC:
+		case zeek::TYPE_FUNC:
 			{
 			sz = SizeAlign(sz, sizeof(uint32_t));
 			break;
 			}
 
-		case TYPE_PATTERN:
+		case zeek::TYPE_PATTERN:
 			{
 			if ( ! v )
 				return (optional && ! calc_static_size) ? sz : 0;
 
-			sz = SizeAlign(sz, 2 * sizeof(size_t));
+			sz = SizeAlign(sz, 2 * sizeof(uint64_t));
 			sz += strlen(v->AsPattern()->PatternText())
 				+ strlen(v->AsPattern()->AnywherePatternText()) + 2; // 2 for null terminators
 			break;
 			}
 
-		case TYPE_RECORD:
+		case zeek::TYPE_RECORD:
 			{
-			const RecordVal* rv = v ? v->AsRecordVal() : 0;
-			RecordType* rt = bt->AsRecordType();
+			const zeek::RecordVal* rv = v ? v->AsRecordVal() : nullptr;
+			zeek::RecordType* rt = bt->AsRecordType();
 			int num_fields = rt->NumFields();
 
 			for ( int i = 0; i < num_fields; ++i )
 				{
-				Attributes* a = rt->FieldDecl(i)->attrs;
-				bool optional = (a && a->FindAttr(ATTR_OPTIONAL));
+				zeek::detail::Attributes* a = rt->FieldDecl(i)->attrs.get();
+				bool optional = (a && a->Find(zeek::detail::ATTR_OPTIONAL));
 
-				sz = SingleTypeKeySize(rt->FieldType(i),
-						       rv ? rv->Lookup(i) : 0,
+				sz = SingleTypeKeySize(rt->GetFieldType(i).get(),
+						       rv ? rv->GetField(i).get() : nullptr,
 						       type_check, sz, optional,
 						       calc_static_size);
 				if ( ! sz )
@@ -532,58 +529,50 @@ int CompositeHash::SingleTypeKeySize(BroType* bt, const Val* v,
 			break;
 			}
 
-		case TYPE_TABLE:
+		case zeek::TYPE_TABLE:
 			{
 			if ( ! v )
 				return (optional && ! calc_static_size) ? sz : 0;
 
 			sz = SizeAlign(sz, sizeof(int));
-			TableVal* tv = const_cast<TableVal*>(v->AsTableVal());
-			ListVal* lv = tv->ConvertToList();
+			zeek::TableVal* tv = const_cast<zeek::TableVal*>(v->AsTableVal());
+			auto lv = tv->ToListVal();
 			for ( int i = 0; i < tv->Size(); ++i )
 				{
-				Val* key = lv->Index(i);
-				sz = SingleTypeKeySize(key->Type(), key, type_check, sz, false,
+				const auto& key = lv->Idx(i);
+				sz = SingleTypeKeySize(key->GetType().get(), key.get(), type_check, sz, false,
 				                       calc_static_size);
 				if ( ! sz )
-					{
-					Unref(lv);
 					return 0;
-					}
 
 				if ( ! bt->IsSet() )
 					{
-					Val* val = tv->Lookup(key);
-					sz = SingleTypeKeySize(val->Type(), val, type_check, sz,
+					auto val = tv->FindOrDefault(key);
+					sz = SingleTypeKeySize(val->GetType().get(), val.get(), type_check, sz,
 					                       false, calc_static_size);
 					if ( ! sz )
-						{
-						Unref(lv);
 						return 0;
-						}
 					}
 				}
 
-			Unref(lv);
-
 			break;
 			}
 
-		case TYPE_VECTOR:
+		case zeek::TYPE_VECTOR:
 			{
 			if ( ! v )
 				return (optional && ! calc_static_size) ? sz : 0;
 
 			sz = SizeAlign(sz, sizeof(unsigned int));
-			VectorVal* vv = const_cast<VectorVal*>(v->AsVectorVal());
+			zeek::VectorVal* vv = const_cast<zeek::VectorVal*>(v->AsVectorVal());
 			for ( unsigned int i = 0; i < vv->Size(); ++i )
 				{
-				Val* val = vv->Lookup(i);
+				const auto& val = vv->At(i);
 				sz = SizeAlign(sz, sizeof(unsigned int));
 				sz = SizeAlign(sz, sizeof(unsigned int));
 				if ( val )
-					sz = SingleTypeKeySize(bt->AsVectorType()->YieldType(),
-					                       val, type_check, sz, false,
+					sz = SingleTypeKeySize(bt->AsVectorType()->Yield().get(),
+					                       val.get(), type_check, sz, false,
 					                       calc_static_size);
 				if ( ! sz ) return 0;
 				}
@@ -591,16 +580,16 @@ int CompositeHash::SingleTypeKeySize(BroType* bt, const Val* v,
 			break;
 			}
 
-		case TYPE_LIST:
+		case zeek::TYPE_LIST:
 			{
 			if ( ! v )
 				return (optional && ! calc_static_size) ? sz : 0;
 
 			sz = SizeAlign(sz, sizeof(int));
-			ListVal* lv = const_cast<ListVal*>(v->AsListVal());
+			zeek::ListVal* lv = const_cast<zeek::ListVal*>(v->AsListVal());
 			for ( int i = 0; i < lv->Length(); ++i )
 				{
-				sz = SingleTypeKeySize(lv->Index(i)->Type(), lv->Index(i),
+				sz = SingleTypeKeySize(lv->Idx(i)->GetType().get(), lv->Idx(i).get(),
 				                       type_check, sz, false, calc_static_size);
 				if ( ! sz) return 0;
 				}
@@ -615,10 +604,10 @@ int CompositeHash::SingleTypeKeySize(BroType* bt, const Val* v,
 			}
 		}
 
-		break; // case TYPE_INTERNAL_VOID/OTHER
+		break; // case zeek::TYPE_INTERNAL_VOID/OTHER
 		}
 
-	case TYPE_INTERNAL_STRING:
+	case zeek::TYPE_INTERNAL_STRING:
 		if ( ! v )
 			return (optional && ! calc_static_size) ? sz : 0;
 
@@ -627,31 +616,32 @@ int CompositeHash::SingleTypeKeySize(BroType* bt, const Val* v,
 		sz += v->AsString()->Len();
 		break;
 
-	case TYPE_INTERNAL_ERROR:
+	case zeek::TYPE_INTERNAL_ERROR:
 		return 0;
 	}
 
 	return sz;
 	}
 
-int CompositeHash::ComputeKeySize(const Val* v, int type_check, bool calc_static_size) const
+int CompositeHash::ComputeKeySize(const zeek::Val* v, bool type_check, bool calc_static_size) const
 	{
-	const type_list* tl = type->Types();
-	const val_list* vl = 0;
+	const auto& tl = type->GetTypes();
+
 	if ( v )
 		{
-		if ( type_check && v->Type()->Tag() != TYPE_LIST )
+		if ( type_check && v->GetType()->Tag() != zeek::TYPE_LIST )
 			return 0;
 
-		vl = v->AsListVal()->Vals();
-		if ( type_check && vl->length() != tl->length() )
+		auto lv = v->AsListVal();
+
+		if ( type_check && lv->Length() != static_cast<int>(tl.size()) )
 			return 0;
 		}
 
 	int sz = 0;
-	loop_over_list(*tl, i)
+	for ( auto i = 0u; i < tl.size(); ++i )
 		{
-		sz = SingleTypeKeySize((*tl)[i], v ? v->AsListVal()->Index(i) : 0,
+		sz = SingleTypeKeySize(tl[i].get(), v ? v->AsListVal()->Idx(i).get() : nullptr,
 				       type_check, sz, false, calc_static_size);
 		if ( ! sz )
 			return 0;
@@ -719,19 +709,19 @@ int CompositeHash::SizeAlign(int offset, unsigned int size) const
 	return offset;
 	}
 
-ListVal* CompositeHash::RecoverVals(const HashKey* k) const
+zeek::ListValPtr CompositeHash::RecoverVals(const HashKey& k) const
 	{
-	ListVal* l = new ListVal(TYPE_ANY);
-	const type_list* tl = type->Types();
-	const char* kp = (const char*) k->Key();
-	const char* const k_end = kp + k->Size();
+	auto l = zeek::make_intrusive<zeek::ListVal>(zeek::TYPE_ANY);
+	const auto& tl = type->GetTypes();
+	const char* kp = (const char*) k.Key();
+	const char* const k_end = kp + k.Size();
 
-	for ( const auto& type : *tl )
+	for ( const auto& type : tl )
 		{
-		Val* v = nullptr;
-		kp = RecoverOneVal(k, kp, k_end, type, v, false);
+		zeek::ValPtr v;
+		kp = RecoverOneVal(k, kp, k_end, type.get(), &v, false);
 		ASSERT(v);
-		l->Append(v);
+		l->Append(std::move(v));
 		}
 
 	if ( kp != k_end )
@@ -740,17 +730,18 @@ ListVal* CompositeHash::RecoverVals(const HashKey* k) const
 	return l;
 	}
 
-const char* CompositeHash::RecoverOneVal(const HashKey* k, const char* kp0,
-					 const char* const k_end, BroType* t,
-					 Val*& pval, bool optional) const
+const char* CompositeHash::RecoverOneVal(
+	const HashKey& k, const char* kp0,
+	const char* const k_end, zeek::Type* t,
+	zeek::ValPtr* pval, bool optional) const
 	{
 	// k->Size() == 0 for a single empty string.
-	if ( kp0 >= k_end && k->Size() > 0 )
+	if ( kp0 >= k_end && k.Size() > 0 )
 		reporter->InternalError("over-ran key in CompositeHash::RecoverVals");
 
-	TypeTag tag = t->Tag();
-	InternalTypeTag it = t->InternalType();
-	const char* kp1 = 0;
+	zeek::TypeTag tag = t->Tag();
+	zeek::InternalTypeTag it = t->InternalType();
+	const char* kp1 = nullptr;
 
 	if ( optional )
 		{
@@ -759,67 +750,69 @@ const char* CompositeHash::RecoverOneVal(const HashKey* k, const char* kp0,
 
 		if ( ! *kp )
 			{
-			pval = 0;
+			*pval = nullptr;
 			return kp0;
 			}
 		}
 
 	switch ( it ) {
-	case TYPE_INTERNAL_INT:
+	case zeek::TYPE_INTERNAL_INT:
 		{
 		const bro_int_t* const kp = AlignType<bro_int_t>(kp0);
 		kp1 = reinterpret_cast<const char*>(kp+1);
 
-		if ( tag == TYPE_ENUM )
-			pval = t->AsEnumType()->GetVal(*kp);
-		else if ( tag == TYPE_BOOL )
-			pval = val_mgr->GetBool(*kp);
-		else if ( tag == TYPE_INT )
-			pval = val_mgr->GetInt(*kp);
+		if ( tag == zeek::TYPE_ENUM )
+			*pval = t->AsEnumType()->GetEnumVal(*kp);
+		else if ( tag == zeek::TYPE_BOOL )
+			*pval = zeek::val_mgr->Bool(*kp);
+		else if ( tag == zeek::TYPE_INT )
+			*pval = zeek::val_mgr->Int(*kp);
 		else
 			{
 			reporter->InternalError("bad internal unsigned int in CompositeHash::RecoverOneVal()");
-			pval = 0;
+			*pval = nullptr;
 			}
 		}
 		break;
 
-	case TYPE_INTERNAL_UNSIGNED:
+	case zeek::TYPE_INTERNAL_UNSIGNED:
 		{
 		const bro_uint_t* const kp = AlignType<bro_uint_t>(kp0);
 		kp1 = reinterpret_cast<const char*>(kp+1);
 
 		switch ( tag ) {
-		case TYPE_COUNT:
-		case TYPE_COUNTER:
-			pval = val_mgr->GetCount(*kp);
+		case zeek::TYPE_COUNT:
+		case zeek::TYPE_COUNTER:
+			*pval = zeek::val_mgr->Count(*kp);
 			break;
 
-		case TYPE_PORT:
-			pval = val_mgr->GetPort(*kp);
+		case zeek::TYPE_PORT:
+			*pval = zeek::val_mgr->Port(*kp);
 			break;
 
 		default:
 			reporter->InternalError("bad internal unsigned int in CompositeHash::RecoverOneVal()");
-			pval = 0;
+			*pval = nullptr;
 			break;
 		}
 		}
 		break;
 
-	case TYPE_INTERNAL_DOUBLE:
+	case zeek::TYPE_INTERNAL_DOUBLE:
 		{
 		const double* const kp = AlignType<double>(kp0);
 		kp1 = reinterpret_cast<const char*>(kp+1);
 
-		if ( tag == TYPE_INTERVAL )
-			pval = new IntervalVal(*kp, 1.0);
+		if ( tag == zeek::TYPE_INTERVAL )
+			*pval = zeek::make_intrusive<zeek::IntervalVal>(*kp, 1.0);
+		else if ( tag == zeek::TYPE_TIME )
+			*pval = zeek::make_intrusive<zeek::TimeVal>(*kp);
 		else
-			pval = new Val(*kp, tag);
+			*pval = zeek::make_intrusive<zeek::DoubleVal>(*kp);
 		}
 		break;
 
-	case TYPE_INTERNAL_ADDR:
+	case zeek::TYPE_INTERNAL_ADDR:
 		{
 		const uint32_t* const kp = AlignType<uint32_t>(kp0);
 		kp1 = reinterpret_cast<const char*>(kp+4);
@@ -827,59 +820,58 @@ const char* CompositeHash::RecoverOneVal(const HashKey* k, const char* kp0,
 		IPAddr addr(IPv6, kp, IPAddr::Network);
 
 		switch ( tag ) {
-		case TYPE_ADDR:
-			pval = new AddrVal(addr);
+		case zeek::TYPE_ADDR:
+			*pval = zeek::make_intrusive<zeek::AddrVal>(addr);
 			break;
 
 		default:
 			reporter->InternalError("bad internal address in CompositeHash::RecoverOneVal()");
-			pval = 0;
+			*pval = nullptr;
 			break;
 		}
 		}
 		break;
 
-	case TYPE_INTERNAL_SUBNET:
+	case zeek::TYPE_INTERNAL_SUBNET:
 		{
 		const uint32_t* const kp = AlignType<uint32_t>(kp0);
 		kp1 = reinterpret_cast<const char*>(kp+5);
-		pval = new SubNetVal(kp, kp[4]);
+		*pval = zeek::make_intrusive<zeek::SubNetVal>(kp, kp[4]);
 		}
 		break;
 
-	case TYPE_INTERNAL_VOID:
-	case TYPE_INTERNAL_OTHER:
+	case zeek::TYPE_INTERNAL_VOID:
+	case zeek::TYPE_INTERNAL_OTHER:
 		{
 		switch ( t->Tag() ) {
-		case TYPE_FUNC:
+		case zeek::TYPE_FUNC:
 			{
 			const uint32_t* const kp = AlignType<uint32_t>(kp0);
 			kp1 = reinterpret_cast<const char*>(kp+1);
 
-			Func* f = Func::GetFuncPtrByID(*kp);
+			const auto& f = zeek::Func::GetFuncPtrByID(*kp);
 
 			if ( ! f )
 				reporter->InternalError("failed to look up unique function id %" PRIu32 " in CompositeHash::RecoverOneVal()", *kp);
 
-			pval = new Val(f);
+			*pval = zeek::make_intrusive<zeek::Val>(f);
+			const auto& pvt = (*pval)->GetType();
 
-			if ( ! pval->Type() )
+			if ( ! pvt )
 				reporter->InternalError("bad aggregate Val in CompositeHash::RecoverOneVal()");
 
-			else if ( t->Tag() != TYPE_FUNC &&
-				  ! same_type(pval->Type(), t) )
+			else if ( t->Tag() != zeek::TYPE_FUNC && ! same_type(pvt, t) )
 				// ### Maybe fix later, but may be fundamentally
 				// un-checkable --US
 				reporter->InternalError("inconsistent aggregate Val in CompositeHash::RecoverOneVal()");
 
 			// ### A crude approximation for now.
-			else if ( t->Tag() == TYPE_FUNC &&
-				  pval->Type()->Tag() != TYPE_FUNC )
+			else if ( t->Tag() == zeek::TYPE_FUNC && pvt->Tag() != zeek::TYPE_FUNC )
 				reporter->InternalError("inconsistent aggregate Val in CompositeHash::RecoverOneVal()");
 			}
 			break;
 
-		case TYPE_PATTERN:
+		case zeek::TYPE_PATTERN:
 			{
 			RE_Matcher* re = nullptr;
 			if ( is_singleton )
@@ -887,11 +879,11 @@ const char* CompositeHash::RecoverOneVal(const HashKey* k, const char* kp0,
 				kp1 = kp0;
 				int divider = strlen(kp0) + 1;
 				re = new RE_Matcher(kp1, kp1 + divider);
-				kp1 += k->Size();
+				kp1 += k.Size();
 				}
 			else
 				{
-				const size_t* const len = AlignType<size_t>(kp0);
+				const uint64_t* const len = AlignType<uint64_t>(kp0);
 
 				kp1 = reinterpret_cast<const char*>(len+2);
 				re = new RE_Matcher(kp1, kp1 + len[0]);
@@ -901,90 +893,92 @@ const char* CompositeHash::RecoverOneVal(const HashKey* k, const char* kp0,
 			if ( ! re->Compile() )
 				reporter->InternalError("failed compiling table/set key pattern: %s",
 				                        re->PatternText());
-			pval = new PatternVal(re);
+
+			*pval = zeek::make_intrusive<zeek::PatternVal>(re);
 			}
 			break;
 
-		case TYPE_RECORD:
+		case zeek::TYPE_RECORD:
 			{
 			const char* kp = kp0;
-			RecordType* rt = t->AsRecordType();
+			zeek::RecordType* rt = t->AsRecordType();
 			int num_fields = rt->NumFields();
 
-			vector<Val*> values;
+			std::vector<zeek::ValPtr> values;
 			int i;
 			for ( i = 0; i < num_fields; ++i )
 				{
-				Val* v;
+				zeek::ValPtr v;
 
-				Attributes* a = rt->FieldDecl(i)->attrs;
-				bool optional = (a && a->FindAttr(ATTR_OPTIONAL));
+				zeek::detail::Attributes* a = rt->FieldDecl(i)->attrs.get();
+				bool optional = (a && a->Find(zeek::detail::ATTR_OPTIONAL));
 
 				kp = RecoverOneVal(k, kp, k_end,
-				                   rt->FieldType(i), v, optional);
+				                   rt->GetFieldType(i).get(), &v, optional);
+
+				// An earlier call to reporter->InternalError would have called abort() and broken the
+				// call tree that clang-tidy is relying on to get the error described.
+				// NOLINTNEXTLINE(clang-analyzer-core.uninitialized.Branch)
 				if ( ! (v || optional) )
 					{
 					reporter->InternalError("didn't recover expected number of fields from HashKey");
-					pval = 0;
+					pval = nullptr;
 					break;
 					}
 
-				values.push_back(v);
+				values.emplace_back(std::move(v));
 				}
 
 			ASSERT(int(values.size()) == num_fields);
 
-			RecordVal* rv = new RecordVal(rt);
+			auto rv = zeek::make_intrusive<zeek::RecordVal>(zeek::IntrusivePtr{zeek::NewRef{}, rt});
 
 			for ( int i = 0; i < num_fields; ++i )
-				rv->Assign(i, values[i]);
+				rv->Assign(i, std::move(values[i]));
 
-			pval = rv;
+			*pval = std::move(rv);
 			kp1 = kp;
 			}
 			break;
 
-		case TYPE_TABLE:
+		case zeek::TYPE_TABLE:
 			{
 			int n;
 			const int* const kp = AlignType<int>(kp0);
 			n = *kp;
 			kp1 = reinterpret_cast<const char*>(kp+1);
-			TableType* tt = t->AsTableType();
-			TableVal* tv = new TableVal(tt);
-			vector<Val*> keys, values;
+			zeek::TableType* tt = t->AsTableType();
+			auto tv = zeek::make_intrusive<zeek::TableVal>(zeek::IntrusivePtr{zeek::NewRef{}, tt});
+
 			for ( int i = 0; i < n; ++i )
 				{
-				Val* key;
-				kp1 = RecoverOneVal(k, kp1, k_end, tt->Indices(), key, false);
-				keys.push_back(key);
-				if ( ! t->IsSet() )
+				zeek::ValPtr key;
+				kp1 = RecoverOneVal(k, kp1, k_end, tt->GetIndices().get(), &key, false);
+
+				if ( t->IsSet() )
+					tv->Assign(std::move(key), nullptr);
+				else
 					{
-					Val* value;
-					kp1 = RecoverOneVal(k, kp1, k_end, tt->YieldType(), value,
+					zeek::ValPtr value;
+					kp1 = RecoverOneVal(k, kp1, k_end, tt->Yield().get(), &value,
 					                    false);
-					values.push_back(value);
+					tv->Assign(std::move(key), std::move(value));
 					}
 				}
 
-			for ( int i = 0; i < n; ++i )
-				{
-				tv->Assign(keys[i], t->IsSet() ? 0 : values[i]);
-				Unref(keys[i]);
-				}
-
-			pval = tv;
+			*pval = std::move(tv);
 			}
 			break;
 
-		case TYPE_VECTOR:
+		case zeek::TYPE_VECTOR:
 			{
 			unsigned int n;
 			const unsigned int* kp = AlignType<unsigned int>(kp0);
 			n = *kp;
 			kp1 = reinterpret_cast<const char*>(kp+1);
-			VectorType* vt = t->AsVectorType();
-			VectorVal* vv = new VectorVal(vt);
+			zeek::VectorType* vt = t->AsVectorType();
+			auto vv = zeek::make_intrusive<zeek::VectorVal>(zeek::IntrusivePtr{zeek::NewRef{}, vt});
+
 			for ( unsigned int i = 0; i < n; ++i )
 				{
 				kp = AlignType<unsigned int>(kp1);
@@ -993,34 +987,37 @@ const char* CompositeHash::RecoverOneVal(const HashKey* k, const char* kp0,
 				kp = AlignType<unsigned int>(kp1);
 				unsigned int have_val = *kp;
 				kp1 = reinterpret_cast<const char*>(kp+1);
-				Val* value = 0;
+				zeek::ValPtr value;
+
 				if ( have_val )
-					kp1 = RecoverOneVal(k, kp1, k_end, vt->YieldType(), value,
+					kp1 = RecoverOneVal(k, kp1, k_end, vt->Yield().get(), &value,
 					                    false);
-				vv->Assign(index, value);
+
+				vv->Assign(index, std::move(value));
 				}
 
-			pval = vv;
+			*pval = std::move(vv);
 			}
 			break;
 
-		case TYPE_LIST:
+		case zeek::TYPE_LIST:
 			{
 			int n;
 			const int* const kp = AlignType<int>(kp0);
 			n = *kp;
 			kp1 = reinterpret_cast<const char*>(kp+1);
-			TypeList* tl = t->AsTypeList();
-			ListVal* lv = new ListVal(TYPE_ANY);
+			zeek::TypeList* tl = t->AsTypeList();
+			auto lv = zeek::make_intrusive<zeek::ListVal>(zeek::TYPE_ANY);
+
 			for ( int i = 0; i < n; ++i )
 				{
-				Val* v;
-				BroType* it = (*tl->Types())[i];
-				kp1 = RecoverOneVal(k, kp1, k_end, it, v, false);
-				lv->Append(v);
+				zeek::ValPtr v;
+				zeek::Type* it = tl->GetTypes()[i].get();
+				kp1 = RecoverOneVal(k, kp1, k_end, it, &v, false);
+				lv->Append(std::move(v));
 				}
 
-			pval = lv;
+			*pval = std::move(lv);
 			}
 			break;
 
@@ -1032,7 +1029,7 @@ const char* CompositeHash::RecoverOneVal(const HashKey* k, const char* kp0,
 		}
 		break;
 
-	case TYPE_INTERNAL_STRING:
+	case zeek::TYPE_INTERNAL_STRING:
 		{
 		// There is a minor issue here -- the pointer does not have to
 		// be aligned by int in the singleton case.
@@ -1041,7 +1038,7 @@ const char* CompositeHash::RecoverOneVal(const HashKey* k, const char* kp0,
 		if ( is_singleton )
 			{
 			kp1 = kp0;
-			n = k->Size();
+			n = k.Size();
 			}
 		else
 			{
@@ -1050,12 +1047,12 @@ const char* CompositeHash::RecoverOneVal(const HashKey* k, const char* kp0,
 			kp1 = reinterpret_cast<const char*>(kp+1);
 			}
 
-		pval = new StringVal(new BroString((const byte_vec) kp1, n, 1));
+		*pval = zeek::make_intrusive<zeek::StringVal>(new zeek::String((const zeek::byte_vec) kp1, n, true));
 		kp1 += n;
 		}
 		break;
 
-	case TYPE_INTERNAL_ERROR:
+	case zeek::TYPE_INTERNAL_ERROR:
 		break;
 	}
 
