@@ -39,7 +39,7 @@ ConnectionTimer::~ConnectionTimer()
 	Unref(conn);
 	}
 
-void ConnectionTimer::Dispatch(double t, int is_expire)
+void ConnectionTimer::Dispatch(double t, bool is_expire)
 	{
 	if ( is_expire && ! do_expire )
 		return;
@@ -80,17 +80,16 @@ Connection::Connection(NetSessions* s, const ConnIDKey& k, double t, const ConnI
 	if ( pkt->l2_src )
 		memcpy(orig_l2_addr, pkt->l2_src, sizeof(orig_l2_addr));
 	else
-		bzero(orig_l2_addr, sizeof(orig_l2_addr));
+		memset(orig_l2_addr, 0, sizeof(orig_l2_addr));
 
 	if ( pkt->l2_dst )
 		memcpy(resp_l2_addr, pkt->l2_dst, sizeof(resp_l2_addr));
 	else
-		bzero(resp_l2_addr, sizeof(resp_l2_addr));
+		memset(resp_l2_addr, 0, sizeof(resp_l2_addr));
 
 	vlan = pkt->vlan;
 	inner_vlan = pkt->inner_vlan;
 
-	conn_val = nullptr;
 	login_conn = nullptr;
 
 	is_active = 1;
@@ -120,7 +119,7 @@ Connection::Connection(NetSessions* s, const ConnIDKey& k, double t, const ConnI
 	if ( arg_encap )
 		encapsulation = new EncapsulationStack(*arg_encap);
 	else
-		encapsulation = 0;
+		encapsulation = nullptr;
 	}
 
 Connection::~Connection()
@@ -131,10 +130,7 @@ Connection::~Connection()
 	CancelTimers();
 
 	if ( conn_val )
-		{
-		conn_val->SetOrigin(0);
-		Unref(conn_val);
-		}
+		conn_val->SetOrigin(nullptr);
 
 	delete root_analyzer;
 	delete encapsulation;
@@ -148,7 +144,10 @@ void Connection::CheckEncapsulation(const EncapsulationStack* arg_encap)
 		{
 		if ( *encapsulation != *arg_encap )
 			{
-			Event(tunnel_changed, 0, arg_encap->GetVectorVal());
+			if ( tunnel_changed )
+				EnqueueEvent(tunnel_changed, nullptr, ConnVal(),
+				             arg_encap->ToVal());
+
 			delete encapsulation;
 			encapsulation = new EncapsulationStack(*arg_encap);
 			}
@@ -156,15 +155,21 @@ void Connection::CheckEncapsulation(const EncapsulationStack* arg_encap)
 
 	else if ( encapsulation )
 		{
-		EncapsulationStack empty;
-		Event(tunnel_changed, 0, empty.GetVectorVal());
+		if ( tunnel_changed )
+			{
+			EncapsulationStack empty;
+			EnqueueEvent(tunnel_changed, nullptr, ConnVal(), empty.ToVal());
+			}
+
 		delete encapsulation;
 		encapsulation = nullptr;
 		}
 
 	else if ( arg_encap )
 		{
-		Event(tunnel_changed, 0, arg_encap->GetVectorVal());
+		if ( tunnel_changed )
+			EnqueueEvent(tunnel_changed, nullptr, ConnVal(), arg_encap->ToVal());
+
 		encapsulation = new EncapsulationStack(*arg_encap);
 		}
 	}
@@ -177,7 +182,7 @@ void Connection::Done()
 		root_analyzer->Done();
 	}
 
-void Connection::NextPacket(double t, int is_orig,
+void Connection::NextPacket(double t, bool is_orig,
 			const IP_Hdr* ip, int len, int caplen,
 			const u_char*& data,
 			int& record_packet, int& record_content,
@@ -203,7 +208,7 @@ void Connection::NextPacket(double t, int is_orig,
 			is_successful = true;
 
 		if ( ! was_successful && is_successful && connection_successful )
-			ConnectionEventFast(connection_successful, nullptr, {BuildConnVal()});
+			EnqueueEvent(connection_successful, nullptr, ConnVal());
 		}
 	else
 		last_time = t;
@@ -259,38 +264,32 @@ void Connection::HistoryThresholdEvent(EventHandlerPtr e, bool is_orig,
 		// and at this stage it's not a *multiple* instance.
 		return;
 
-	ConnectionEventFast(e, 0, {
-		BuildConnVal(),
-		val_mgr->GetBool(is_orig),
-		val_mgr->GetCount(threshold)
-	});
+	EnqueueEvent(e, nullptr,
+		ConnVal(),
+		zeek::val_mgr->Bool(is_orig),
+		zeek::val_mgr->Count(threshold)
+	);
 	}
 
 void Connection::DeleteTimer(double /* t */)
 	{
 	if ( is_active )
-		Event(connection_timeout, 0);
+		Event(connection_timeout, nullptr);
 
 	sessions->Remove(this);
 	}
 
 void Connection::InactivityTimer(double t)
 	{
-	// If the inactivity_timeout is zero, there has been an active
-	// timeout once, but it's disabled now. We do nothing then.
-	if ( inactivity_timeout )
+	if ( last_time + inactivity_timeout <= t )
 		{
-		if ( last_time + inactivity_timeout <= t )
-			{
-			Event(connection_timeout, 0);
-			sessions->Remove(this);
-			++killed_by_inactivity;
-			}
-		else
-			ADD_TIMER(&Connection::InactivityTimer,
-					last_time + inactivity_timeout, 0,
-					TIMER_CONN_INACTIVITY);
+		Event(connection_timeout, nullptr);
+		sessions->Remove(this);
+		++killed_by_inactivity;
 		}
+	else
+		ADD_TIMER(&Connection::InactivityTimer,
+		          last_time + inactivity_timeout, 0, TIMER_CONN_INACTIVITY);
 	}
 
 void Connection::RemoveConnectionTimer(double t)
@@ -301,8 +300,17 @@ void Connection::RemoveConnectionTimer(double t)
 
 void Connection::SetInactivityTimeout(double timeout)
 	{
-	// We add a new inactivity timer even if there already is one.  When
-	// it fires, we always use the current value to check for inactivity.
+	if ( timeout == inactivity_timeout )
+		return;
+
+	// First cancel and remove any existing inactivity timer.
+	for ( const auto& timer : timers )
+		if ( timer->Type() == TIMER_CONN_INACTIVITY )
+			{
+			timer_mgr->Cancel(timer);
+			break;
+			}
+
 	if ( timeout )
 		ADD_TIMER(&Connection::InactivityTimer,
 				last_time + timeout, 0, TIMER_CONN_INACTIVITY);
@@ -323,91 +331,94 @@ void Connection::EnableStatusUpdateTimer()
 
 void Connection::StatusUpdateTimer(double t)
 	{
-	ConnectionEventFast(connection_status_update, 0, { BuildConnVal() });
+	EnqueueEvent(connection_status_update, nullptr, ConnVal());
 	ADD_TIMER(&Connection::StatusUpdateTimer,
 			network_time + connection_status_update_interval, 0,
 			TIMER_CONN_STATUS_UPDATE);
 	}
 
-RecordVal* Connection::BuildConnVal()
+zeek::RecordVal* Connection::BuildConnVal()
+	{
+	return ConnVal()->Ref()->AsRecordVal();
+	}
+
+const zeek::RecordValPtr& Connection::ConnVal()
 	{
 	if ( ! conn_val )
 		{
-		conn_val = new RecordVal(connection_type);
+		conn_val = zeek::make_intrusive<zeek::RecordVal>(zeek::id::connection);
 
 		TransportProto prot_type = ConnTransport();
 
-		RecordVal* id_val = new RecordVal(conn_id);
-		id_val->Assign(0, new AddrVal(orig_addr));
-		id_val->Assign(1, val_mgr->GetPort(ntohs(orig_port), prot_type));
-		id_val->Assign(2, new AddrVal(resp_addr));
-		id_val->Assign(3, val_mgr->GetPort(ntohs(resp_port), prot_type));
+		auto id_val = zeek::make_intrusive<zeek::RecordVal>(zeek::id::conn_id);
+		id_val->Assign(0, zeek::make_intrusive<zeek::AddrVal>(orig_addr));
+		id_val->Assign(1, zeek::val_mgr->Port(ntohs(orig_port), prot_type));
+		id_val->Assign(2, zeek::make_intrusive<zeek::AddrVal>(resp_addr));
+		id_val->Assign(3, zeek::val_mgr->Port(ntohs(resp_port), prot_type));
 
-		RecordVal* orig_endp = new RecordVal(endpoint);
-		orig_endp->Assign(0, val_mgr->GetCount(0));
-		orig_endp->Assign(1, val_mgr->GetCount(0));
-		orig_endp->Assign(4, val_mgr->GetCount(orig_flow_label));
+		auto orig_endp = zeek::make_intrusive<zeek::RecordVal>(zeek::id::endpoint);
+		orig_endp->Assign(0, zeek::val_mgr->Count(0));
+		orig_endp->Assign(1, zeek::val_mgr->Count(0));
+		orig_endp->Assign(4, zeek::val_mgr->Count(orig_flow_label));
 
 		const int l2_len = sizeof(orig_l2_addr);
 		char null[l2_len]{};
 
 		if ( memcmp(&orig_l2_addr, &null, l2_len) != 0 )
-			orig_endp->Assign(5, new StringVal(fmt_mac(orig_l2_addr, l2_len)));
+			orig_endp->Assign(5, zeek::make_intrusive<zeek::StringVal>(fmt_mac(orig_l2_addr, l2_len)));
 
-		RecordVal* resp_endp = new RecordVal(endpoint);
-		resp_endp->Assign(0, val_mgr->GetCount(0));
-		resp_endp->Assign(1, val_mgr->GetCount(0));
-		resp_endp->Assign(4, val_mgr->GetCount(resp_flow_label));
+		auto resp_endp = zeek::make_intrusive<zeek::RecordVal>(zeek::id::endpoint);
+		resp_endp->Assign(0, zeek::val_mgr->Count(0));
+		resp_endp->Assign(1, zeek::val_mgr->Count(0));
+		resp_endp->Assign(4, zeek::val_mgr->Count(resp_flow_label));
 
 		if ( memcmp(&resp_l2_addr, &null, l2_len) != 0 )
-			resp_endp->Assign(5, new StringVal(fmt_mac(resp_l2_addr, l2_len)));
+			resp_endp->Assign(5, zeek::make_intrusive<zeek::StringVal>(fmt_mac(resp_l2_addr, l2_len)));
 
-		conn_val->Assign(0, id_val);
-		conn_val->Assign(1, orig_endp);
-		conn_val->Assign(2, resp_endp);
+		conn_val->Assign(0, std::move(id_val));
+		conn_val->Assign(1, std::move(orig_endp));
+		conn_val->Assign(2, std::move(resp_endp));
 		// 3 and 4 are set below.
-		conn_val->Assign(5, new TableVal(string_set));	// service
-		conn_val->Assign(6, val_mgr->GetEmptyString());	// history
+		conn_val->Assign(5, zeek::make_intrusive<zeek::TableVal>(zeek::id::string_set));	// service
+		conn_val->Assign(6, zeek::val_mgr->EmptyString());	// history
 
 		if ( ! uid )
 			uid.Set(bits_per_uid);
 
-		conn_val->Assign(7, new StringVal(uid.Base62("C").c_str()));
+		conn_val->Assign(7, zeek::make_intrusive<zeek::StringVal>(uid.Base62("C").c_str()));
 
 		if ( encapsulation && encapsulation->Depth() > 0 )
-			conn_val->Assign(8, encapsulation->GetVectorVal());
+			conn_val->Assign(8, encapsulation->ToVal());
 
 		if ( vlan != 0 )
-			conn_val->Assign(9, val_mgr->GetInt(vlan));
+			conn_val->Assign(9, zeek::val_mgr->Int(vlan));
 
 		if ( inner_vlan != 0 )
-			conn_val->Assign(10, val_mgr->GetInt(inner_vlan));
+			conn_val->Assign(10, zeek::val_mgr->Int(inner_vlan));
 
 		}
 
 	if ( root_analyzer )
-		root_analyzer->UpdateConnVal(conn_val);
+		root_analyzer->UpdateConnVal(conn_val.get());
 
-	conn_val->Assign(3, new Val(start_time, TYPE_TIME));	// ###
-	conn_val->Assign(4, new Val(last_time - start_time, TYPE_INTERVAL));
-	conn_val->Assign(6, new StringVal(history.c_str()));
-	conn_val->Assign(11, val_mgr->GetBool(is_successful));
+	conn_val->Assign(3, zeek::make_intrusive<zeek::TimeVal>(start_time));	// ###
+	conn_val->Assign(4, zeek::make_intrusive<zeek::IntervalVal>(last_time - start_time));
+	conn_val->Assign(6, zeek::make_intrusive<zeek::StringVal>(history.c_str()));
+	conn_val->Assign(11, zeek::val_mgr->Bool(is_successful));
 
 	conn_val->SetOrigin(this);
 
-	Ref(conn_val);
-
 	return conn_val;
 	}
 
 analyzer::Analyzer* Connection::FindAnalyzer(analyzer::ID id)
 	{
-	return root_analyzer ? root_analyzer->FindChild(id) : 0;
+	return root_analyzer ? root_analyzer->FindChild(id) : nullptr;
 	}
 
-analyzer::Analyzer* Connection::FindAnalyzer(analyzer::Tag tag)
+analyzer::Analyzer* Connection::FindAnalyzer(const analyzer::Tag& tag)
 	{
-	return root_analyzer ? root_analyzer->FindChild(tag) : 0;
+	return root_analyzer ? root_analyzer->FindChild(tag) : nullptr;
 	}
 
 analyzer::Analyzer* Connection::FindAnalyzer(const char* name)
@@ -417,12 +428,12 @@ analyzer::Analyzer* Connection::FindAnalyzer(const char* name)
 
 void Connection::AppendAddl(const char* str)
 	{
-	Unref(BuildConnVal());
+	const auto& cv = ConnVal();
 
-	const char* old = conn_val->Lookup(6)->AsString()->CheckString();
+	const char* old = cv->GetField(6)->AsString()->CheckString();
 	const char* format = *old ? "%s %s" : "%s%s";
 
-	conn_val->Assign(6, new StringVal(fmt(format, old, str)));
+	cv->Assign(6, zeek::make_intrusive<zeek::StringVal>(fmt(format, old, str)));
 	}
 
 // Returns true if the character at s separates a version number.
@@ -446,15 +457,11 @@ void Connection::Match(Rule::PatternType type, const u_char* data, int len, bool
 
 void Connection::RemovalEvent()
 	{
-	auto cv = BuildConnVal();
-
 	if ( connection_state_remove )
-		ConnectionEventFast(connection_state_remove, nullptr, {cv->Ref()});
+		EnqueueEvent(connection_state_remove, nullptr, ConnVal());
 
 	if ( is_successful && successful_connection_remove )
-		ConnectionEventFast(successful_connection_remove, nullptr, {cv->Ref()});
-
-	Unref(cv);
+		EnqueueEvent(successful_connection_remove, nullptr, ConnVal());
 	}
 
 void Connection::Event(EventHandlerPtr f, analyzer::Analyzer* analyzer, const char* name)
@@ -463,13 +470,12 @@ void Connection::Event(EventHandlerPtr f, analyzer::Analyzer* analyzer, const ch
 		return;
 
 	if ( name )
-		ConnectionEventFast(f, analyzer, {new StringVal(name), BuildConnVal()});
+		EnqueueEvent(f, analyzer, zeek::make_intrusive<zeek::StringVal>(name), ConnVal());
 	else
-		ConnectionEventFast(f, analyzer, {BuildConnVal()});
-
+		EnqueueEvent(f, analyzer, ConnVal());
 	}
 
-void Connection::Event(EventHandlerPtr f, analyzer::Analyzer* analyzer, Val* v1, Val* v2)
+void Connection::Event(EventHandlerPtr f, analyzer::Analyzer* analyzer, zeek::Val* v1, zeek::Val* v2)
 	{
 	if ( ! f )
 		{
@@ -479,39 +485,50 @@ void Connection::Event(EventHandlerPtr f, analyzer::Analyzer* analyzer, Val* v1,
 		}
 
 	if ( v2 )
-		ConnectionEventFast(f, analyzer, {BuildConnVal(), v1, v2});
+		EnqueueEvent(f, analyzer,
+		             ConnVal(),
+		             zeek::IntrusivePtr{zeek::AdoptRef{}, v1},
+		             zeek::IntrusivePtr{zeek::AdoptRef{}, v2});
 	else
-		ConnectionEventFast(f, analyzer, {BuildConnVal(), v1});
+		EnqueueEvent(f, analyzer,
+		             ConnVal(),
+		             zeek::IntrusivePtr{zeek::AdoptRef{}, v1});
 	}
 
 void Connection::ConnectionEvent(EventHandlerPtr f, analyzer::Analyzer* a, val_list vl)
 	{
+	auto args = zeek::val_list_to_args(vl);
+
 	if ( ! f )
-		{
 		// This may actually happen if there is no local handler
 		// and a previously existing remote handler went away.
-		for ( const auto& v : vl)
-			Unref(v);
-
 		return;
-		}
 
 	// "this" is passed as a cookie for the event
-	mgr.QueueEvent(f, std::move(vl), SOURCE_LOCAL,
-			a ? a->GetID() : 0, timer_mgr, this);
+	mgr.Enqueue(f, std::move(args), SOURCE_LOCAL, a ? a->GetID() : 0, this);
 	}
 
 void Connection::ConnectionEventFast(EventHandlerPtr f, analyzer::Analyzer* a, val_list vl)
 	{
 	// "this" is passed as a cookie for the event
-	mgr.QueueEventFast(f, std::move(vl), SOURCE_LOCAL,
-			a ? a->GetID() : 0, timer_mgr, this);
+	mgr.Enqueue(f, zeek::val_list_to_args(vl), SOURCE_LOCAL,
+	            a ? a->GetID() : 0, this);
 	}
 
 void Connection::ConnectionEvent(EventHandlerPtr f, analyzer::Analyzer* a, val_list* vl)
 	{
-	ConnectionEvent(f, a, std::move(*vl));
+	auto args = zeek::val_list_to_args(*vl);
 	delete vl;
+
+	if ( f )
+		EnqueueEvent(f, a, std::move(args));
+	}
+
+void Connection::EnqueueEvent(EventHandlerPtr f, analyzer::Analyzer* a,
+                              zeek::Args args)
+	{
+	// "this" is passed as a cookie for the event
+	mgr.Enqueue(f, std::move(args), SOURCE_LOCAL, a ? a->GetID() : 0, this);
 	}
 
 void Connection::Weird(const char* name, const char* addl)
@@ -582,7 +599,6 @@ void Connection::FlipRoles()
 	resp_flow_label = orig_flow_label;
 	orig_flow_label = tmp_flow;
 
-	Unref(conn_val);
 	conn_val = nullptr;
 
 	if ( root_analyzer )
@@ -681,19 +697,19 @@ void Connection::CheckFlowLabel(bool is_orig, uint32_t flow_label)
 		{
 		if ( conn_val )
 			{
-			RecordVal *endp = conn_val->Lookup(is_orig ? 1 : 2)->AsRecordVal();
-			endp->Assign(4, val_mgr->GetCount(flow_label));
+			zeek::RecordVal* endp = conn_val->GetField(is_orig ? 1 : 2)->AsRecordVal();
+			endp->Assign(4, zeek::val_mgr->Count(flow_label));
 			}
 
 		if ( connection_flow_label_changed &&
 		     (is_orig ? saw_first_orig_packet : saw_first_resp_packet) )
 			{
-			ConnectionEventFast(connection_flow_label_changed, 0, {
-				BuildConnVal(),
-				val_mgr->GetBool(is_orig),
-				val_mgr->GetCount(my_flow_label),
-				val_mgr->GetCount(flow_label),
-			});
+			EnqueueEvent(connection_flow_label_changed, nullptr,
+				ConnVal(),
+				zeek::val_mgr->Bool(is_orig),
+				zeek::val_mgr->Count(my_flow_label),
+				zeek::val_mgr->Count(flow_label)
+			);
 			}
 
 		my_flow_label = flow_label;
