@@ -3,8 +3,10 @@
 #include <memory>
 
 #include "OpaqueVal.h"
+#include "CompHash.h"
 #include "NetVar.h"
 #include "Reporter.h"
+#include "Scope.h"
 #include "Desc.h"
 #include "Var.h"
 #include "probabilistic/BloomFilter.h"
@@ -13,6 +15,8 @@
 #include <broker/data.hh>
 #include <broker/error.hh>
 
+namespace zeek {
+
 // Helper to retrieve a broker value out of a broker::vector at a specified
 // index, and casted to the expected destination type.
 template<typename S, typename V, typename D>
@@ -35,9 +39,11 @@ OpaqueMgr* OpaqueMgr::mgr()
 	return &mgr;
 	}
 
-OpaqueVal::OpaqueVal(OpaqueType* t) : Val(t)
-	{
-	}
+OpaqueVal::OpaqueVal(zeek::OpaqueType* t) : OpaqueVal({zeek::NewRef{}, t})
+	{}
+
+OpaqueVal::OpaqueVal(zeek::OpaqueTypePtr t) : Val(std::move(t))
+	{}
 
 OpaqueVal::~OpaqueVal()
 	{
@@ -54,7 +60,7 @@ const std::string& OpaqueMgr::TypeID(const OpaqueVal* v) const
 	return x->first;
 	}
 
-OpaqueVal* OpaqueMgr::Instantiate(const std::string& id) const
+OpaqueValPtr OpaqueMgr::Instantiate(const std::string& id) const
 	{
 	auto x = _types.find(id);
 	return x != _types.end() ? (*x->second)() : nullptr;
@@ -71,7 +77,7 @@ broker::expected<broker::data> OpaqueVal::Serialize() const
 	return {broker::vector{std::move(type), std::move(*d)}};
 	}
 
-OpaqueVal* OpaqueVal::Unserialize(const broker::data& data)
+OpaqueValPtr OpaqueVal::Unserialize(const broker::data& data)
 	{
 	auto v = caf::get_if<broker::vector>(&data);
 
@@ -87,20 +93,17 @@ OpaqueVal* OpaqueVal::Unserialize(const broker::data& data)
 		return nullptr;
 
 	if ( ! val->DoUnserialize((*v)[1]) )
-		{
-		Unref(val);
 		return nullptr;
-		}
 
 	return val;
 	}
 
-broker::expected<broker::data> OpaqueVal::SerializeType(BroType* t)
+broker::expected<broker::data> OpaqueVal::SerializeType(const zeek::TypePtr& t)
 	{
-	if ( t->InternalType() == TYPE_INTERNAL_ERROR )
+	if ( t->InternalType() == zeek::TYPE_INTERNAL_ERROR )
 		return broker::ec::invalid_data;
 
-	if ( t->InternalType() == TYPE_INTERNAL_OTHER )
+	if ( t->InternalType() == zeek::TYPE_INTERNAL_OTHER )
 		{
 		// Serialize by name.
 		assert(t->GetName().size());
@@ -111,7 +114,7 @@ broker::expected<broker::data> OpaqueVal::SerializeType(BroType* t)
 	return {broker::vector{false, static_cast<uint64_t>(t->Tag())}};
 	}
 
-BroType* OpaqueVal::UnserializeType(const broker::data& data)
+zeek::TypePtr OpaqueVal::UnserializeType(const broker::data& data)
 	{
 	auto v = caf::get_if<broker::vector>(&data);
 	if ( ! (v && v->size() == 2) )
@@ -127,32 +130,31 @@ BroType* OpaqueVal::UnserializeType(const broker::data& data)
 		if ( ! name )
 			return nullptr;
 
-		ID* id = global_scope()->Lookup(*name);
+		const auto& id = zeek::detail::global_scope()->Find(*name);
 		if ( ! id )
 			return nullptr;
 
-		BroType* t = id->AsType();
-		if ( ! t )
+		if ( ! id->IsType() )
 			return nullptr;
 
-		return t->Ref();
+		return id->GetType();
 		}
 
 	auto tag = caf::get_if<uint64_t>(&(*v)[1]);
 	if ( ! tag )
 		return nullptr;
 
-	return base_type(static_cast<TypeTag>(*tag));
+	return zeek::base_type(static_cast<zeek::TypeTag>(*tag));
 	}
 
-Val* OpaqueVal::DoClone(CloneState* state)
+ValPtr OpaqueVal::DoClone(CloneState* state)
 	{
 	auto d = OpaqueVal::Serialize();
 	if ( ! d )
 		return nullptr;
 
 	auto rval = OpaqueVal::Unserialize(std::move(*d));
-	return state->NewClone(this, rval);
+	return state->NewClone(this, std::move(rval));
 	}
 
 bool HashVal::IsValid() const
@@ -169,12 +171,12 @@ bool HashVal::Init()
 	return valid;
 	}
 
-StringVal* HashVal::Get()
+StringValPtr HashVal::Get()
 	{
 	if ( ! valid )
-		return val_mgr->GetEmptyString();
+		return zeek::val_mgr->EmptyString();
 
-	StringVal* result = DoGet();
+	auto result = DoGet();
 	valid = false;
 	return result;
 	}
@@ -200,17 +202,20 @@ bool HashVal::DoFeed(const void*, size_t)
 	return false;
 	}
 
-StringVal* HashVal::DoGet()
+StringValPtr HashVal::DoGet()
 	{
 	assert(! "missing implementation of DoGet()");
-	return val_mgr->GetEmptyString();
+	return zeek::val_mgr->EmptyString();
 	}
 
-HashVal::HashVal(OpaqueType* t) : OpaqueVal(t)
+HashVal::HashVal(zeek::OpaqueTypePtr t) : OpaqueVal(std::move(t))
 	{
 	valid = false;
 	}
 
+HashVal::HashVal(zeek::OpaqueType* t) : HashVal({zeek::NewRef{}, t})
+	{}
+
 MD5Val::MD5Val() : HashVal(md5_type)
 	{
 	}
@@ -221,50 +226,39 @@ MD5Val::~MD5Val()
 		EVP_MD_CTX_free(ctx);
 	}
 
-Val* MD5Val::DoClone(CloneState* state)
+void HashVal::digest_one(EVP_MD_CTX* h, const Val* v)
 	{
-	auto out = new MD5Val();
+	if ( v->GetType()->Tag() == zeek::TYPE_STRING )
+		{
+		const String* str = v->AsString();
+		hash_update(h, str->Bytes(), str->Len());
+		}
+	else
+		{
+		ODesc d(DESC_BINARY);
+		v->Describe(&d);
+		hash_update(h, (const u_char *) d.Bytes(), d.Len());
+		}
+	}
+
+void HashVal::digest_one(EVP_MD_CTX* h, const ValPtr& v)
+	{
+	digest_one(h, v.get());
+	}
+
+ValPtr MD5Val::DoClone(CloneState* state)
+	{
+	auto out = zeek::make_intrusive<MD5Val>();
+
 	if ( IsValid() )
 		{
 		if ( ! out->Init() )
 			return nullptr;
+
 		EVP_MD_CTX_copy_ex(out->ctx, ctx);
 		}
 
-	return state->NewClone(this, out);
-	}
-
-void MD5Val::digest(val_list& vlist, u_char result[MD5_DIGEST_LENGTH])
-	{
-	EVP_MD_CTX* h = hash_init(Hash_MD5);
-
-	for ( const auto& v : vlist )
-		{
-		if ( v->Type()->Tag() == TYPE_STRING )
-			{
-			const BroString* str = v->AsString();
-			hash_update(h, str->Bytes(), str->Len());
-			}
-		else
-			{
-			ODesc d(DESC_BINARY);
-			v->Describe(&d);
-			hash_update(h, (const u_char *) d.Bytes(), d.Len());
-			}
-		}
-
-	hash_final(h, result);
-	}
-
-void MD5Val::hmac(val_list& vlist,
-                  u_char key[MD5_DIGEST_LENGTH],
-                  u_char result[MD5_DIGEST_LENGTH])
-	{
-	digest(vlist, result);
-	for ( int i = 0; i < MD5_DIGEST_LENGTH; ++i )
-		result[i] ^= key[i];
-
-	internal_md5(result, MD5_DIGEST_LENGTH, result);
+	return state->NewClone(this, std::move(out));
 	}
 
 bool MD5Val::DoInit()
@@ -283,14 +277,14 @@ bool MD5Val::DoFeed(const void* data, size_t size)
 	return true;
 	}
 
-StringVal* MD5Val::DoGet()
+StringValPtr MD5Val::DoGet()
 	{
 	if ( ! IsValid() )
-		return val_mgr->GetEmptyString();
+		return zeek::val_mgr->EmptyString();
 
 	u_char digest[MD5_DIGEST_LENGTH];
 	hash_final(ctx, digest);
-	return new StringVal(md5_digest_print(digest));
+	return zeek::make_intrusive<StringVal>(md5_digest_print(digest));
 	}
 
 IMPLEMENT_OPAQUE_VALUE(MD5Val)
@@ -372,39 +366,19 @@ SHA1Val::~SHA1Val()
 		EVP_MD_CTX_free(ctx);
 	}
 
-Val* SHA1Val::DoClone(CloneState* state)
+ValPtr SHA1Val::DoClone(CloneState* state)
 	{
-	auto out = new SHA1Val();
+	auto out = zeek::make_intrusive<SHA1Val>();
+
 	if ( IsValid() )
 		{
 		if ( ! out->Init() )
 			return nullptr;
+
 		EVP_MD_CTX_copy_ex(out->ctx, ctx);
 		}
 
-	return state->NewClone(this, out);
-	}
-
-void SHA1Val::digest(val_list& vlist, u_char result[SHA_DIGEST_LENGTH])
-	{
-	EVP_MD_CTX* h = hash_init(Hash_SHA1);
-
-	for ( const auto& v : vlist )
-		{
-		if ( v->Type()->Tag() == TYPE_STRING )
-			{
-			const BroString* str = v->AsString();
-			hash_update(h, str->Bytes(), str->Len());
-			}
-		else
-			{
-			ODesc d(DESC_BINARY);
-			v->Describe(&d);
-			hash_update(h, (const u_char *) d.Bytes(), d.Len());
-			}
-		}
-
-	hash_final(h, result);
+	return state->NewClone(this, std::move(out));
 	}
 
 bool SHA1Val::DoInit()
@@ -423,14 +397,14 @@ bool SHA1Val::DoFeed(const void* data, size_t size)
 	return true;
 	}
 
-StringVal* SHA1Val::DoGet()
+StringValPtr SHA1Val::DoGet()
 	{
 	if ( ! IsValid() )
-		return val_mgr->GetEmptyString();
+		return zeek::val_mgr->EmptyString();
 
 	u_char digest[SHA_DIGEST_LENGTH];
 	hash_final(ctx, digest);
-	return new StringVal(sha1_digest_print(digest));
+	return zeek::make_intrusive<StringVal>(sha1_digest_print(digest));
 	}
 
 IMPLEMENT_OPAQUE_VALUE(SHA1Val)
@@ -515,39 +489,19 @@ SHA256Val::~SHA256Val()
 		EVP_MD_CTX_free(ctx);
 	}
 
-Val* SHA256Val::DoClone(CloneState* state)
+ValPtr SHA256Val::DoClone(CloneState* state)
 	{
-	auto out = new SHA256Val();
+	auto out = zeek::make_intrusive<SHA256Val>();
+
 	if ( IsValid() )
 		{
 		if ( ! out->Init() )
 			return nullptr;
+
 		EVP_MD_CTX_copy_ex(out->ctx, ctx);
 		}
 
-	return state->NewClone(this, out);
-	}
-
-void SHA256Val::digest(val_list& vlist, u_char result[SHA256_DIGEST_LENGTH])
-	{
-	EVP_MD_CTX* h = hash_init(Hash_SHA256);
-
-	for ( const auto& v : vlist )
-		{
-		if ( v->Type()->Tag() == TYPE_STRING )
-			{
-			const BroString* str = v->AsString();
-			hash_update(h, str->Bytes(), str->Len());
-			}
-		else
-			{
-			ODesc d(DESC_BINARY);
-			v->Describe(&d);
-			hash_update(h, (const u_char *) d.Bytes(), d.Len());
-			}
-		}
-
-	hash_final(h, result);
+	return state->NewClone(this, std::move(out));
 	}
 
 bool SHA256Val::DoInit()
@@ -566,14 +520,14 @@ bool SHA256Val::DoFeed(const void* data, size_t size)
 	return true;
 	}
 
-StringVal* SHA256Val::DoGet()
+StringValPtr SHA256Val::DoGet()
 	{
 	if ( ! IsValid() )
-		return val_mgr->GetEmptyString();
+		return zeek::val_mgr->EmptyString();
 
 	u_char digest[SHA256_DIGEST_LENGTH];
 	hash_final(ctx, digest);
-	return new StringVal(sha256_digest_print(digest));
+	return zeek::make_intrusive<StringVal>(sha256_digest_print(digest));
 	}
 
 IMPLEMENT_OPAQUE_VALUE(SHA256Val)
@@ -748,72 +702,53 @@ bool EntropyVal::DoUnserialize(const broker::data& data)
 BloomFilterVal::BloomFilterVal()
 	: OpaqueVal(bloomfilter_type)
 	{
-	type = 0;
-	hash = 0;
-	bloom_filter = 0;
-	}
-
-BloomFilterVal::BloomFilterVal(OpaqueType* t)
-	: OpaqueVal(t)
-	{
-	type = 0;
-	hash = 0;
-	bloom_filter = 0;
+	hash = nullptr;
+	bloom_filter = nullptr;
 	}
 
 BloomFilterVal::BloomFilterVal(probabilistic::BloomFilter* bf)
 	: OpaqueVal(bloomfilter_type)
 	{
-	type = 0;
-	hash = 0;
+	hash = nullptr;
 	bloom_filter = bf;
 	}
 
-Val* BloomFilterVal::DoClone(CloneState* state)
+ValPtr BloomFilterVal::DoClone(CloneState* state)
 	{
 	if ( bloom_filter )
 		{
-		auto bf = new BloomFilterVal(bloom_filter->Clone());
+		auto bf = zeek::make_intrusive<BloomFilterVal>(bloom_filter->Clone());
 		bf->Typify(type);
-		return state->NewClone(this, bf);
+		return state->NewClone(this, std::move(bf));
 		}
 
-	return state->NewClone(this, new BloomFilterVal());
+	return state->NewClone(this, zeek::make_intrusive<BloomFilterVal>());
 	}
 
-bool BloomFilterVal::Typify(BroType* arg_type)
+bool BloomFilterVal::Typify(zeek::TypePtr arg_type)
 	{
 	if ( type )
 		return false;
 
-	type = arg_type;
-	type->Ref();
+	type = std::move(arg_type);
 
-	TypeList* tl = new TypeList(type);
-	tl->Append(type->Ref());
-	hash = new CompositeHash(tl);
-	Unref(tl);
+	auto tl = zeek::make_intrusive<zeek::TypeList>(type);
+	tl->Append(type);
+	hash = new CompositeHash(std::move(tl));
 
 	return true;
 	}
 
-BroType* BloomFilterVal::Type() const
-	{
-	return type;
-	}
-
 void BloomFilterVal::Add(const Val* val)
 	{
-	HashKey* key = hash->ComputeHash(val, 1);
-	bloom_filter->Add(key);
-	delete key;
+	auto key = hash->MakeHashKey(*val, true);
+	bloom_filter->Add(key.get());
 	}
 
 size_t BloomFilterVal::Count(const Val* val) const
 	{
-	HashKey* key = hash->ComputeHash(val, 1);
-	size_t cnt = bloom_filter->Count(key);
-	delete key;
+	auto key = hash->MakeHashKey(*val, true);
+	size_t cnt = bloom_filter->Count(key.get());
 	return cnt;
 	}
 
@@ -827,42 +762,43 @@ bool BloomFilterVal::Empty() const
 	return bloom_filter->Empty();
 	}
 
-string BloomFilterVal::InternalState() const
+std::string BloomFilterVal::InternalState() const
 	{
 	return bloom_filter->InternalState();
 	}
 
-BloomFilterVal* BloomFilterVal::Merge(const BloomFilterVal* x,
-				      const BloomFilterVal* y)
+BloomFilterValPtr BloomFilterVal::Merge(const BloomFilterVal* x,
+                                        const BloomFilterVal* y)
 	{
 	if ( x->Type() && // any one 0 is ok here
 	     y->Type() &&
 	     ! same_type(x->Type(), y->Type()) )
 		{
 		reporter->Error("cannot merge Bloom filters with different types");
-		return 0;
+		return nullptr;
 		}
 
 	if ( typeid(*x->bloom_filter) != typeid(*y->bloom_filter) )
 		{
 		reporter->Error("cannot merge different Bloom filter types");
-		return 0;
+		return nullptr;
 		}
 
 	probabilistic::BloomFilter* copy = x->bloom_filter->Clone();
 
 	if ( ! copy->Merge(y->bloom_filter) )
 		{
+		delete copy;
 		reporter->Error("failed to merge Bloom filter");
-		return 0;
+		return nullptr;
 		}
 
-	BloomFilterVal* merged = new BloomFilterVal(copy);
+	auto merged = zeek::make_intrusive<BloomFilterVal>(copy);
 
 	if ( x->Type() && ! merged->Typify(x->Type()) )
 		{
 		reporter->Error("failed to set type on merged Bloom filter");
-		return 0;
+		return nullptr;
 		}
 
 	return merged;
@@ -870,7 +806,6 @@ BloomFilterVal* BloomFilterVal::Merge(const BloomFilterVal* x,
 
 BloomFilterVal::~BloomFilterVal()
 	{
-	Unref(type);
 	delete hash;
 	delete bloom_filter;
 	}
@@ -910,8 +845,9 @@ bool BloomFilterVal::DoUnserialize(const broker::data& data)
 	auto no_type = caf::get_if<broker::none>(&(*v)[0]);
 	if ( ! no_type )
 		{
-		BroType* t = UnserializeType((*v)[0]);
-		if ( ! (t && Typify(t)) )
+		auto t = UnserializeType((*v)[0]);
+
+		if ( ! (t && Typify(std::move(t))) )
 			return false;
 		}
 
@@ -925,58 +861,47 @@ bool BloomFilterVal::DoUnserialize(const broker::data& data)
 
 CardinalityVal::CardinalityVal() : OpaqueVal(cardinality_type)
 	{
-	c = 0;
-	type = 0;
-	hash = 0;
+	c = nullptr;
+	hash = nullptr;
 	}
 
 CardinalityVal::CardinalityVal(probabilistic::CardinalityCounter* arg_c)
 	: OpaqueVal(cardinality_type)
 	{
 	c = arg_c;
-	type = 0;
-	hash = 0;
+	hash = nullptr;
 	}
 
 CardinalityVal::~CardinalityVal()
 	{
-	Unref(type);
 	delete c;
 	delete hash;
 	}
 
-Val* CardinalityVal::DoClone(CloneState* state)
+ValPtr CardinalityVal::DoClone(CloneState* state)
 	{
 	return state->NewClone(this,
-			       new CardinalityVal(new probabilistic::CardinalityCounter(*c)));
+			       zeek::make_intrusive<CardinalityVal>(new probabilistic::CardinalityCounter(*c)));
 	}
 
-bool CardinalityVal::Typify(BroType* arg_type)
+bool CardinalityVal::Typify(zeek::TypePtr arg_type)
 	{
 	if ( type )
 		return false;
 
-	type = arg_type;
-	type->Ref();
+	type = std::move(arg_type);
 
-	TypeList* tl = new TypeList(type);
-	tl->Append(type->Ref());
-	hash = new CompositeHash(tl);
-	Unref(tl);
+	auto tl = zeek::make_intrusive<zeek::TypeList>(type);
+	tl->Append(type);
+	hash = new CompositeHash(std::move(tl));
 
 	return true;
 	}
 
-BroType* CardinalityVal::Type() const
-	{
-	return type;
-	}
-
 void CardinalityVal::Add(const Val* val)
 	{
-	HashKey* key = hash->ComputeHash(val, 1);
+	auto key = hash->MakeHashKey(*val, true);
 	c->AddElement(key->Hash());
-	delete key;
 	}
 
 IMPLEMENT_OPAQUE_VALUE(CardinalityVal)
@@ -1014,8 +939,9 @@ bool CardinalityVal::DoUnserialize(const broker::data& data)
 	auto no_type = caf::get_if<broker::none>(&(*v)[0]);
 	if ( ! no_type )
 		{
-		BroType* t = UnserializeType((*v)[0]);
-                if ( ! (t && Typify(t)) )
+		auto t = UnserializeType((*v)[0]);
+
+		if ( ! (t && Typify(std::move(t))) )
 			return false;
 		}
 
@@ -1033,14 +959,14 @@ ParaglobVal::ParaglobVal(std::unique_ptr<paraglob::Paraglob> p)
 	this->internal_paraglob = std::move(p);
 	}
 
-VectorVal* ParaglobVal::Get(StringVal* &pattern)
+VectorValPtr ParaglobVal::Get(StringVal* &pattern)
 	{
-	VectorVal* rval = new VectorVal(internal_type("string_vec")->AsVectorType());
+	auto rval = zeek::make_intrusive<VectorVal>(zeek::id::string_vec);
 	std::string string_pattern (reinterpret_cast<const char*>(pattern->Bytes()), pattern->Len());
 
 	std::vector<std::string> matches = this->internal_paraglob->get(string_pattern);
 	for (unsigned int i = 0; i < matches.size(); i++)
-		rval->Assign(i, new StringVal(matches.at(i)));
+		rval->Assign(i, zeek::make_intrusive<StringVal>(matches.at(i)));
 
 	return rval;
 	}
@@ -1094,10 +1020,10 @@ bool ParaglobVal::DoUnserialize(const broker::data& data)
 	return true;
 	}
 
-Val* ParaglobVal::DoClone(CloneState* state)
+ValPtr ParaglobVal::DoClone(CloneState* state)
 	{
 	try {
-		return new ParaglobVal
+		return zeek::make_intrusive<ParaglobVal>
 			(std::make_unique<paraglob::Paraglob>(this->internal_paraglob->serialize()));
 		}
 	catch (const paraglob::underflow_error& e)
@@ -1111,3 +1037,5 @@ Val* ParaglobVal::DoClone(CloneState* state)
 		return nullptr;
 		}
 	}
+
+}
