@@ -15,6 +15,7 @@
 #include "Net.h"
 #include "Conn.h"
 #include "Timer.h"
+#include "ID.h"
 #include "EventHandler.h"
 #include "plugin/Plugin.h"
 #include "plugin/Manager.h"
@@ -29,10 +30,11 @@ int closelog();
 }
 #endif
 
-Reporter* reporter = 0;
+Reporter* reporter = nullptr;
 
-Reporter::Reporter()
+Reporter::Reporter(bool arg_abort_on_scripting_errors)
 	{
+	abort_on_scripting_errors = arg_abort_on_scripting_errors;
 	errors = 0;
 	via_events = false;
 	in_error_handler = 0;
@@ -61,25 +63,24 @@ Reporter::~Reporter()
 
 void Reporter::InitOptions()
 	{
-	info_to_stderr = internal_val("Reporter::info_to_stderr")->AsBool();
-	warnings_to_stderr = internal_val("Reporter::warnings_to_stderr")->AsBool();
-	errors_to_stderr = internal_val("Reporter::errors_to_stderr")->AsBool();
-	weird_sampling_rate = internal_val("Weird::sampling_rate")->AsCount();
-	weird_sampling_threshold = internal_val("Weird::sampling_threshold")->AsCount();
-	weird_sampling_duration = internal_val("Weird::sampling_duration")->AsInterval();
-	auto wl_val = internal_val("Weird::sampling_whitelist")->AsTableVal();
+	info_to_stderr = zeek::id::find_val("Reporter::info_to_stderr")->AsBool();
+	warnings_to_stderr = zeek::id::find_val("Reporter::warnings_to_stderr")->AsBool();
+	errors_to_stderr = zeek::id::find_val("Reporter::errors_to_stderr")->AsBool();
+	weird_sampling_rate = zeek::id::find_val("Weird::sampling_rate")->AsCount();
+	weird_sampling_threshold = zeek::id::find_val("Weird::sampling_threshold")->AsCount();
+	weird_sampling_duration = zeek::id::find_val("Weird::sampling_duration")->AsInterval();
+	auto wl_val = zeek::id::find_val("Weird::sampling_whitelist")->AsTableVal();
 	auto wl_table = wl_val->AsTable();
 
 	HashKey* k;
-	IterCookie* c = wl_table->InitForIteration();
-	TableEntryVal* v;
+	zeek::IterCookie* c = wl_table->InitForIteration();
+	zeek::TableEntryVal* v;
 
 	while ( (v = wl_table->NextEntry(k, c)) )
 		{
-		auto index = wl_val->RecoverIndex(k);
-		string key = index->Index(0)->AsString()->CheckString();
+		auto index = wl_val->RecreateIndex(*k);
+		std::string key = index->Idx(0)->AsString()->CheckString();
 		weird_sampling_whitelist.emplace(move(key));
-		Unref(index);
 		delete k;
 		}
 	}
@@ -88,8 +89,8 @@ void Reporter::Info(const char* fmt, ...)
 	{
 	va_list ap;
 	va_start(ap, fmt);
-	FILE* out = EmitToStderr(info_to_stderr) ? stderr : 0;
-	DoLog("", reporter_info, out, 0, 0, true, true, 0, fmt, ap);
+	FILE* out = EmitToStderr(info_to_stderr) ? stderr : nullptr;
+	DoLog("", reporter_info, out, nullptr, nullptr, true, true, nullptr, fmt, ap);
 	va_end(ap);
 	}
 
@@ -97,8 +98,8 @@ void Reporter::Warning(const char* fmt, ...)
 	{
 	va_list ap;
 	va_start(ap, fmt);
-	FILE* out = EmitToStderr(warnings_to_stderr) ? stderr : 0;
-	DoLog("warning", reporter_warning, out, 0, 0, true, true, 0, fmt, ap);
+	FILE* out = EmitToStderr(warnings_to_stderr) ? stderr : nullptr;
+	DoLog("warning", reporter_warning, out, nullptr, nullptr, true, true, nullptr, fmt, ap);
 	va_end(ap);
 	}
 
@@ -107,8 +108,8 @@ void Reporter::Error(const char* fmt, ...)
 	++errors;
 	va_list ap;
 	va_start(ap, fmt);
-	FILE* out = EmitToStderr(errors_to_stderr) ? stderr : 0;
-	DoLog("error", reporter_error, out, 0, 0, true, true, 0, fmt, ap);
+	FILE* out = EmitToStderr(errors_to_stderr) ? stderr : nullptr;
+	DoLog("error", reporter_error, out, nullptr, nullptr, true, true, nullptr, fmt, ap);
 	va_end(ap);
 	}
 
@@ -118,7 +119,7 @@ void Reporter::FatalError(const char* fmt, ...)
 	va_start(ap, fmt);
 
 	// Always log to stderr.
-	DoLog("fatal error", 0, stderr, 0, 0, true, false, 0, fmt, ap);
+	DoLog("fatal error", nullptr, stderr, nullptr, nullptr, true, false, nullptr, fmt, ap);
 
 	va_end(ap);
 
@@ -134,7 +135,7 @@ void Reporter::FatalErrorWithCore(const char* fmt, ...)
 	va_start(ap, fmt);
 
 	// Always log to stderr.
-	DoLog("fatal error", 0, stderr, 0, 0, true, false, 0, fmt, ap);
+	DoLog("fatal error", nullptr, stderr, nullptr, nullptr, true, false, nullptr, fmt, ap);
 
 	va_end(ap);
 
@@ -142,7 +143,7 @@ void Reporter::FatalErrorWithCore(const char* fmt, ...)
 	abort();
 	}
 
-void Reporter::ExprRuntimeError(const Expr* expr, const char* fmt, ...)
+void Reporter::ExprRuntimeError(const zeek::detail::Expr* expr, const char* fmt, ...)
 	{
 	++errors;
 
@@ -152,24 +153,32 @@ void Reporter::ExprRuntimeError(const Expr* expr, const char* fmt, ...)
 	PushLocation(expr->GetLocationInfo());
 	va_list ap;
 	va_start(ap, fmt);
-	FILE* out = EmitToStderr(errors_to_stderr) ? stderr : 0;
-	DoLog("expression error", reporter_error, out, 0, 0, true, true,
+	FILE* out = EmitToStderr(errors_to_stderr) ? stderr : nullptr;
+	DoLog("expression error", reporter_error, out, nullptr, nullptr, true, true,
 	      d.Description(), fmt, ap);
 	va_end(ap);
 	PopLocation();
+
+	if ( abort_on_scripting_errors )
+		abort();
+
 	throw InterpreterException();
 	}
 
-void Reporter::RuntimeError(const Location* location, const char* fmt, ...)
+void Reporter::RuntimeError(const zeek::detail::Location* location, const char* fmt, ...)
 	{
 	++errors;
 	PushLocation(location);
 	va_list ap;
 	va_start(ap, fmt);
-	FILE* out = EmitToStderr(errors_to_stderr) ? stderr : 0;
-	DoLog("runtime error", reporter_error, out, 0, 0, true, true, "", fmt, ap);
+	FILE* out = EmitToStderr(errors_to_stderr) ? stderr : nullptr;
+	DoLog("runtime error", reporter_error, out, nullptr, nullptr, true, true, "", fmt, ap);
 	va_end(ap);
 	PopLocation();
+
+	if ( abort_on_scripting_errors )
+		abort();
+
 	throw InterpreterException();
 	}
 
@@ -179,7 +188,7 @@ void Reporter::InternalError(const char* fmt, ...)
 	va_start(ap, fmt);
 
 	// Always log to stderr.
-	DoLog("internal error", 0, stderr, 0, 0, true, false, 0, fmt, ap);
+	DoLog("internal error", nullptr, stderr, nullptr, nullptr, true, false, nullptr, fmt, ap);
 
 	va_end(ap);
 
@@ -197,7 +206,7 @@ void Reporter::AnalyzerError(analyzer::Analyzer* a, const char* fmt,
 	va_start(ap, fmt);
 	// Always log to stderr.
 	// TODO: would be nice to also log a call stack.
-	DoLog("analyzer error", reporter_error, stderr, 0, 0, true, true, 0, fmt,
+	DoLog("analyzer error", reporter_error, stderr, nullptr, nullptr, true, true, nullptr, fmt,
 	      ap);
 	va_end(ap);
 	}
@@ -206,9 +215,9 @@ void Reporter::InternalWarning(const char* fmt, ...)
 	{
 	va_list ap;
 	va_start(ap, fmt);
-	FILE* out = EmitToStderr(warnings_to_stderr) ? stderr : 0;
+	FILE* out = EmitToStderr(warnings_to_stderr) ? stderr : nullptr;
 	// TODO: would be nice to also log a call stack.
-	DoLog("internal warning", reporter_warning, out, 0, 0, true, true, 0, fmt,
+	DoLog("internal warning", reporter_warning, out, nullptr, nullptr, true, true, nullptr, fmt,
 	      ap);
 	va_end(ap);
 	}
@@ -228,7 +237,7 @@ void Reporter::WeirdHelper(EventHandlerPtr event, val_list vl, const char* fmt_n
 	{
 	va_list ap;
 	va_start(ap, fmt_name);
-	DoLog("weird", event, 0, 0, &vl, false, false, 0, fmt_name, ap);
+	DoLog("weird", event, nullptr, nullptr, &vl, false, false, nullptr, fmt_name, ap);
 	va_end(ap);
 	}
 
@@ -238,32 +247,47 @@ void Reporter::UpdateWeirdStats(const char* name)
 	++weird_count_by_type[name];
 	}
 
-class NetWeirdTimer : public Timer {
+class NetWeirdTimer final : public Timer {
 public:
 	NetWeirdTimer(double t, const char* name, double timeout)
 	: Timer(t + timeout, TIMER_NET_WEIRD_EXPIRE), weird_name(name)
 		{}
 
-	void Dispatch(double t, int is_expire) override
+	void Dispatch(double t, bool is_expire) override
 		{ reporter->ResetNetWeird(weird_name); }
 
 	std::string weird_name;
 };
 
-class FlowWeirdTimer : public Timer {
+class FlowWeirdTimer final : public Timer {
 public:
 	using IPPair = std::pair<IPAddr, IPAddr>;
 
 	FlowWeirdTimer(double t, IPPair p, double timeout)
-	: Timer(t + timeout, TIMER_FLOW_WEIRD_EXPIRE), endpoints(p)
+		: Timer(t + timeout, TIMER_FLOW_WEIRD_EXPIRE), endpoints(std::move(p))
 		{}
 
-	void Dispatch(double t, int is_expire) override
+	void Dispatch(double t, bool is_expire) override
 		{ reporter->ResetFlowWeird(endpoints.first, endpoints.second); }
 
 	IPPair endpoints;
 };
 
+class ConnTupleWeirdTimer final : public Timer {
+public:
+	using ConnTuple = Reporter::ConnTuple;
+
+	ConnTupleWeirdTimer(double t, ConnTuple id, double timeout)
+		: Timer(t + timeout, TIMER_CONN_TUPLE_WEIRD_EXPIRE),
+		  conn_id(std::move(id))
+		{}
+
+	void Dispatch(double t, bool is_expire) override
+		{ reporter->ResetExpiredConnWeird(conn_id); }
+
+	ConnTuple conn_id;
+};
+
 void Reporter::ResetNetWeird(const std::string& name)
 	{
 	net_weird_state.erase(name);
@@ -274,6 +298,11 @@ void Reporter::ResetFlowWeird(const IPAddr& orig, const IPAddr& resp)
 	flow_weird_state.erase(std::make_pair(orig, resp));
 	}
 
+void Reporter::ResetExpiredConnWeird(const ConnTuple& id)
+	{
+	expired_conn_weird_state.erase(id);
+	}
+
 bool Reporter::PermitNetWeird(const char* name)
 	{
 	auto& count = net_weird_state[name];
@@ -316,6 +345,35 @@ bool Reporter::PermitFlowWeird(const char* name,
 		return false;
 	}
 
+bool Reporter::PermitExpiredConnWeird(const char* name, const zeek::RecordVal& conn_id)
+	{
+	auto conn_tuple = std::make_tuple(conn_id.GetField("orig_h")->AsAddr(),
+	                                  conn_id.GetField("resp_h")->AsAddr(),
+	                                  conn_id.GetField("orig_p")->AsPortVal()->Port(),
+	                                  conn_id.GetField("resp_p")->AsPortVal()->Port(),
+	                                  conn_id.GetField("resp_p")->AsPortVal()->PortType());
+
+	auto& map = expired_conn_weird_state[conn_tuple];
+
+	if ( map.empty() )
+		timer_mgr->Add(new ConnTupleWeirdTimer(network_time,
+		                                       std::move(conn_tuple),
+		                                       weird_sampling_duration));
+
+	auto& count = map[name];
+	++count;
+
+	if ( count <= weird_sampling_threshold )
+		return true;
+
+	auto num_above_threshold = count - weird_sampling_threshold;
+
+	if ( weird_sampling_rate )
+		return num_above_threshold % weird_sampling_rate == 0;
+	else
+		return false;
+	}
+
 void Reporter::Weird(const char* name, const char* addl)
 	{
 	UpdateWeirdStats(name);
@@ -326,7 +384,7 @@ void Reporter::Weird(const char* name, const char* addl)
 			return;
 		}
 
-	WeirdHelper(net_weird, {new StringVal(addl)}, "%s", name);
+	WeirdHelper(net_weird, {new zeek::StringVal(addl)}, "%s", name);
 	}
 
 void Reporter::Weird(file_analysis::File* f, const char* name, const char* addl)
@@ -340,7 +398,7 @@ void Reporter::Weird(file_analysis::File* f, const char* name, const char* addl)
 			return;
 		}
 
-	WeirdHelper(file_weird, {f->GetVal()->Ref(), new StringVal(addl)},
+	WeirdHelper(file_weird, {f->ToVal()->Ref(), new zeek::StringVal(addl)},
 	            "%s", name);
 	}
 
@@ -355,7 +413,23 @@ void Reporter::Weird(Connection* conn, const char* name, const char* addl)
 			return;
 		}
 
-	WeirdHelper(conn_weird, {conn->BuildConnVal(), new StringVal(addl)},
+	WeirdHelper(conn_weird, {conn->ConnVal()->Ref(), new zeek::StringVal(addl)},
+	            "%s", name);
+	}
+
+void Reporter::Weird(zeek::RecordValPtr conn_id, zeek::StringValPtr uid,
+                     const char* name, const char* addl)
+	{
+	UpdateWeirdStats(name);
+
+	if ( ! WeirdOnSamplingWhiteList(name) )
+		{
+		if ( ! PermitExpiredConnWeird(name, *conn_id) )
+			return;
+		}
+
+	WeirdHelper(expired_conn_weird,
+	            {conn_id.release(), uid.release(), new zeek::StringVal(addl)},
 	            "%s", name);
 	}
 
@@ -370,7 +444,7 @@ void Reporter::Weird(const IPAddr& orig, const IPAddr& resp, const char* name, c
 		}
 
 	WeirdHelper(flow_weird,
-	            {new AddrVal(orig), new AddrVal(resp), new StringVal(addl)},
+	            {new zeek::AddrVal(orig), new zeek::AddrVal(resp), new zeek::StringVal(addl)},
 	            "%s", name);
 	}
 
@@ -382,24 +456,24 @@ void Reporter::DoLog(const char* prefix, EventHandlerPtr event, FILE* out,
 
 	int size = sizeof(tmp);
 	char* buffer  = tmp;
-	char* alloced = 0;
+	char* alloced = nullptr;
 
-	string loc_str;
+	std::string loc_str;
 
 	if ( location )
 		{
-		string loc_file = "";
+		std::string loc_file = "";
 		int loc_line = 0;
 
 		if ( locations.size() )
 			{
 			ODesc d;
 
-			std::pair<const Location*, const Location*> locs = locations.back();
+			std::pair<const zeek::detail::Location*, const zeek::detail::Location*> locs = locations.back();
 
 			if ( locs.first )
 				{
-				if ( locs.first != &no_location )
+				if ( locs.first != &zeek::detail::no_location )
 					locs.first->Describe(&d);
 
 				else
@@ -409,7 +483,7 @@ void Reporter::DoLog(const char* prefix, EventHandlerPtr event, FILE* out,
 					{
 					d.Add(" and ");
 
-					if ( locs.second != &no_location )
+					if ( locs.second != &zeek::detail::no_location )
 						locs.second->Describe(&d);
 
 					else
@@ -427,7 +501,7 @@ void Reporter::DoLog(const char* prefix, EventHandlerPtr event, FILE* out,
 			loc_str = filename;
 			char tmp[32];
 			snprintf(tmp, 32, "%d", line_number);
-			loc_str += string(", line ") + string(tmp);
+			loc_str += std::string(", line ") + std::string(tmp);
 			}
 		}
 
@@ -480,26 +554,28 @@ void Reporter::DoLog(const char* prefix, EventHandlerPtr event, FILE* out,
 		auto vl_size = 1 + (bool)time + (bool)location + (bool)conn +
 		               (addl ? addl->length() : 0);
 
-		val_list vl(vl_size);
+		zeek::Args vl;
+		vl.reserve(vl_size);
 
 		if ( time )
-			vl.push_back(new Val(network_time ? network_time : current_time(), TYPE_TIME));
+			vl.emplace_back(zeek::make_intrusive<zeek::TimeVal>(network_time ? network_time : current_time()));
 
-		vl.push_back(new StringVal(buffer));
+		vl.emplace_back(zeek::make_intrusive<zeek::StringVal>(buffer));
 
 		if ( location )
-			vl.push_back(new StringVal(loc_str.c_str()));
+			vl.emplace_back(zeek::make_intrusive<zeek::StringVal>(loc_str.c_str()));
 
 		if ( conn )
-			vl.push_back(conn->BuildConnVal());
+			vl.emplace_back(conn->ConnVal());
 
 		if ( addl )
-			std::copy(addl->begin(), addl->end(), std::back_inserter(vl));
+			for ( auto v : *addl )
+				vl.emplace_back(zeek::AdoptRef{}, v);
 
 		if ( conn )
-			conn->ConnectionEventFast(event, 0, std::move(vl));
+			conn->EnqueueEvent(event, nullptr, std::move(vl));
 		else
-			mgr.QueueEventFast(event, std::move(vl));
+			mgr.Enqueue(event, std::move(vl));
 		}
 	else
 		{
@@ -512,21 +588,21 @@ void Reporter::DoLog(const char* prefix, EventHandlerPtr event, FILE* out,
 
 	if ( out )
 		{
-		string s = "";
+		std::string s = "";
 
 		if ( bro_start_network_time != 0.0 )
 			{
 			char tmp[32];
 			snprintf(tmp, 32, "%.6f", network_time);
-			s += string(tmp) + " ";
+			s += std::string(tmp) + " ";
 			}
 
 		if ( prefix && *prefix )
 			{
 			if ( loc_str != "" )
-				s += string(prefix) + " in " + loc_str + ": ";
+				s += std::string(prefix) + " in " + loc_str + ": ";
 			else
-				s += string(prefix) + ": ";
+				s += std::string(prefix) + ": ";
 			}
 
 		else
