@@ -1,11 +1,28 @@
+#include "Trigger.h"
+
 #include <algorithm>
 
-#include "Trigger.h"
+#include <assert.h>
+
 #include "Traverse.h"
+#include "Expr.h"
+#include "Frame.h"
+#include "ID.h"
+#include "Val.h"
+#include "Stmt.h"
+#include "Reporter.h"
+#include "Desc.h"
+#include "DebugLogger.h"
+#include "iosource/Manager.h"
+
+using namespace zeek::detail;
+using namespace zeek::detail::trigger;
 
 // Callback class to traverse an expression, registering all relevant IDs and
 // Vals for change notifications.
 
+namespace zeek::detail::trigger {
+
 class TriggerTraversalCallback : public TraversalCallback {
 public:
 	TriggerTraversalCallback(Trigger *arg_trigger)
@@ -14,13 +31,13 @@ public:
 	~TriggerTraversalCallback()
 		{ Unref(trigger); }
 
-	virtual TraversalCode PreExpr(const Expr*);
+	virtual TraversalCode PreExpr(const zeek::detail::Expr*) override;
 
 private:
 	Trigger* trigger;
 };
 
-TraversalCode TriggerTraversalCallback::PreExpr(const Expr* expr)
+TraversalCode zeek::detail::trigger::TriggerTraversalCallback::PreExpr(const zeek::detail::Expr* expr)
 	{
 	// We catch all expressions here which in some way reference global
 	// state.
@@ -28,11 +45,12 @@ TraversalCode TriggerTraversalCallback::PreExpr(const Expr* expr)
 	switch ( expr->Tag() ) {
 	case EXPR_NAME:
 		{
-		const NameExpr* e = static_cast<const NameExpr*>(expr);
+		const auto* e = static_cast<const zeek::detail::NameExpr*>(expr);
 		if ( e->Id()->IsGlobal() )
 			trigger->Register(e->Id());
 
-		Val* v = e->Id()->ID_Val();
+		Val* v = e->Id()->GetVal().get();
+
 		if ( v && v->Modifiable() )
 			trigger->Register(v);
 		break;
@@ -40,18 +58,15 @@ TraversalCode TriggerTraversalCallback::PreExpr(const Expr* expr)
 
 	case EXPR_INDEX:
 		{
-		const IndexExpr* e = static_cast<const IndexExpr*>(expr);
-		BroObj::SuppressErrors no_errors;
+		const auto* e = static_cast<const zeek::detail::IndexExpr*>(expr);
+		Obj::SuppressErrors no_errors;
 
 		try
 			{
-			Val* v = e->Eval(trigger->frame);
+			auto v = e->Eval(trigger->frame);
 
 			if ( v )
-				{
-				trigger->Register(v);
-				Unref(v);
-				}
+				trigger->Register(v.get());
 			}
 		catch ( InterpreterException& )
 			{ /* Already reported */ }
@@ -67,7 +82,7 @@ TraversalCode TriggerTraversalCallback::PreExpr(const Expr* expr)
 	return TC_CONTINUE;
 	}
 
-class TriggerTimer : public Timer {
+class TriggerTimer final : public Timer {
 public:
 	TriggerTimer(double arg_timeout, Trigger* arg_trigger)
 	: Timer(network_time + arg_timeout, TIMER_TRIGGER)
@@ -81,7 +96,7 @@ public:
 	~TriggerTimer()
 		{ Unref(trigger); }
 
-	void Dispatch(double t, int is_expire)
+	void Dispatch(double t, bool is_expire) override
 		{
 		// The network_time may still have been zero when the
 		// timer was instantiated.  In this case, it fires
@@ -102,28 +117,24 @@ protected:
 	double time;
 };
 
-Trigger::Trigger(Expr* arg_cond, Stmt* arg_body, Stmt* arg_timeout_stmts,
-			Expr* arg_timeout, Frame* arg_frame,
+Trigger::Trigger(zeek::detail::Expr* arg_cond, zeek::detail::Stmt* arg_body,
+			zeek::detail::Stmt* arg_timeout_stmts,
+			zeek::detail::Expr* arg_timeout, Frame* arg_frame,
 			bool arg_is_return, const Location* arg_location)
 	{
-	if ( ! pending )
-		pending = new list<Trigger*>;
-
 	cond = arg_cond;
 	body = arg_body;
 	timeout_stmts = arg_timeout_stmts;
 	timeout = arg_timeout;
 	frame = arg_frame->Clone();
-	timer = 0;
+	timer = nullptr;
 	delayed = false;
 	disabled = false;
-	attached = 0;
+	attached = nullptr;
 	is_return = arg_is_return;
 	location = arg_location;
 	timeout_value = -1;
 
-	++total_triggers;
-
 	DBG_LOG(DBG_NOTIFIERS, "%s: instantiating", Name());
 
 	if ( is_return )
@@ -140,7 +151,7 @@ Trigger::Trigger(Expr* arg_cond, Stmt* arg_body, Stmt* arg_timeout_stmts,
 		arg_frame->SetDelayed();
 		}
 
-	Val* timeout_val = nullptr;
+	ValPtr timeout_val;
 
 	if ( arg_timeout )
 		{
@@ -155,7 +166,6 @@ Trigger::Trigger(Expr* arg_cond, Stmt* arg_body, Stmt* arg_timeout_stmts,
 	if ( timeout_val )
 		{
 		timeout_value = timeout_val->AsInterval();
-		Unref(timeout_val);
 		}
 
 	// Make sure we don't get deleted if somebody calls a method like
@@ -171,6 +181,27 @@ Trigger::Trigger(Expr* arg_cond, Stmt* arg_body, Stmt* arg_timeout_stmts,
 	Unref(this);
 	}
 
+void Trigger::Terminate()
+	{
+	if ( is_return )
+		{
+		auto parent = frame->GetTrigger();
+
+		if ( ! parent->Disabled() )
+			{
+			// If the trigger was already disabled due to interpreter
+			// exception, an Unref already happened at that point.
+			parent->Disable();
+			Unref(parent);
+			}
+
+		frame->ClearTrigger();
+		}
+
+	Disable();
+	Unref(this);
+	}
+
 Trigger::~Trigger()
 	{
 	DBG_LOG(DBG_NOTIFIERS, "%s: deleting", Name());
@@ -194,9 +225,6 @@ void Trigger::Init()
 	cond->Traverse(&cb);
 	}
 
-Trigger::TriggerList* Trigger::pending = 0;
-unsigned long Trigger::total_triggers = 0;
-
 bool Trigger::Eval()
 	{
 	if ( disabled )
@@ -234,9 +262,9 @@ bool Trigger::Eval()
 		return false;
 		}
 
-	f->SetTrigger(this);
+	f->SetTrigger({zeek::NewRef{}, this});
 
-	Val* v = nullptr;
+	ValPtr v;
 
 	try
 		{
@@ -259,7 +287,6 @@ bool Trigger::Eval()
 		{
 		// Not true. Perhaps next time...
 		DBG_LOG(DBG_NOTIFIERS, "%s: trigger condition is false", Name());
-		Unref(v);
 		Unref(f);
 		Init();
 		return false;
@@ -268,8 +295,7 @@ bool Trigger::Eval()
 	DBG_LOG(DBG_NOTIFIERS, "%s: trigger condition is true, executing",
 			Name());
 
-	Unref(v);
-	v = 0;
+	v = nullptr;
 	stmt_flow_type flow;
 
 	try
@@ -292,12 +318,16 @@ bool Trigger::Eval()
 		delete [] pname;
 #endif
 
-		trigger->Cache(frame->GetCall(), v);
+		auto queued = trigger->Cache(frame->GetCall(), v.get());
 		trigger->Release();
 		frame->ClearTrigger();
+
+		if ( ! queued && trigger->TimeoutValue() < 0 )
+			// Usually the parent-trigger would get unref'd either by
+			// its Eval() or its eventual Timeout(), but has neither
+			Unref(trigger);
 		}
 
-	Unref(v);
 	Unref(f);
 
 	if ( timer )
@@ -309,47 +339,6 @@ bool Trigger::Eval()
 	return true;
 	}
 
-void Trigger::QueueTrigger(Trigger* trigger)
-	{
-	assert(! trigger->disabled);
-	assert(pending);
-	if ( std::find(pending->begin(), pending->end(), trigger) == pending->end() )
-		{
-		Ref(trigger);
-		pending->push_back(trigger);
-		}
-	}
-
-void Trigger::EvaluatePending()
-	{
-	DBG_LOG(DBG_NOTIFIERS, "evaluating all pending triggers");
-
-	if ( ! pending )
-		return;
-
-	// While we iterate over the list, executing statements, we may
-	// in fact trigger new triggers and thereby modify the list.
-	// Therefore, we create a new temporary list which will receive
-	// triggers triggered during this time.
-	TriggerList* orig = pending;
-	TriggerList tmp;
-	pending = &tmp;
-
-	for ( TriggerList::iterator i = orig->begin(); i != orig->end(); ++i )
-		{
-		Trigger* t = *i;
-		(*i)->Eval();
-		Unref(t);
-		}
-
-	pending = orig;
-	orig->clear();
-
-	// Sigh... Is this really better than a for-loop?
-	std::copy(tmp.begin(), tmp.end(),
-		insert_iterator<TriggerList>(*pending, pending->begin()));
-	}
-
 void Trigger::Timeout()
 	{
 	if ( disabled )
@@ -359,12 +348,12 @@ void Trigger::Timeout()
 	if ( timeout_stmts )
 		{
 		stmt_flow_type flow;
-		Frame* f = frame->Clone();
-		Val* v = 0;
+		FramePtr f{zeek::AdoptRef{}, frame->Clone()};
+		ValPtr v;
 
 		try
 			{
-			v = timeout_stmts->Exec(f, flow);
+			v = timeout_stmts->Exec(f.get(), flow);
 			}
 		catch ( InterpreterException& e )
 			{ /* Already reported. */ }
@@ -381,20 +370,22 @@ void Trigger::Timeout()
 			DBG_LOG(DBG_NOTIFIERS, "%s: trigger has parent %s, caching timeout result", Name(), pname);
 			delete [] pname;
 #endif
-			trigger->Cache(frame->GetCall(), v);
+			auto queued = trigger->Cache(frame->GetCall(), v.get());
 			trigger->Release();
 			frame->ClearTrigger();
-			}
 
-		Unref(v);
-		Unref(f);
+			if ( ! queued && trigger->TimeoutValue() < 0 )
+				// Usually the parent-trigger would get unref'd either by
+				// its Eval() or its eventual Timeout(), but has neither
+				Unref(trigger);
+			}
 		}
 
 	Disable();
 	Unref(this);
 	}
 
-void Trigger::Register(ID* id)
+void Trigger::Register(zeek::detail::ID* id)
 	{
 	assert(! disabled);
 	notifier::registry.Register(id, this);
@@ -445,10 +436,10 @@ void Trigger::Attach(Trigger *trigger)
 	Hold();
 	}
 
-void Trigger::Cache(const CallExpr* expr, Val* v)
+bool Trigger::Cache(const zeek::detail::CallExpr* expr, Val* v)
 	{
 	if ( disabled || ! v )
-		return;
+		return false;
 
 	ValCache::iterator i = cache.find(expr);
 
@@ -463,11 +454,12 @@ void Trigger::Cache(const CallExpr* expr, Val* v)
 
 	Ref(v);
 
-	QueueTrigger(this);
+	trigger_mgr->Queue(this);
+	return true;
 	}
 
 
-Val* Trigger::Lookup(const CallExpr* expr)
+Val* Trigger::Lookup(const zeek::detail::CallExpr* expr)
 	{
 	assert(! disabled);
 
@@ -481,6 +473,16 @@ void Trigger::Disable()
 	disabled = true;
 	}
 
+void Trigger::Describe(ODesc* d) const
+	{
+	d->Add("<trigger>");
+	}
+
+void Trigger::Modified(notifier::Modifiable* m)
+	{
+	trigger_mgr->Queue(this);
+	}
+
 const char* Trigger::Name() const
 	{
 	assert(location);
@@ -488,8 +490,64 @@ const char* Trigger::Name() const
 			location->first_line, location->last_line);
 	}
 
-void Trigger::GetStats(Stats* stats)
+
+
+Manager::Manager() : IOSource()
+	{
+	pending = new TriggerList();
+	iosource_mgr->Register(this, true);
+	}
+
+Manager::~Manager()
+	{
+	delete pending;
+	}
+
+double Manager::GetNextTimeout()
+	{
+	return pending->empty() ? -1 : network_time + 0.100;
+	}
+
+void Manager::Process()
+	{
+	DBG_LOG(DBG_NOTIFIERS, "evaluating all pending triggers");
+
+	// While we iterate over the list, executing statements, we may
+	// in fact trigger new triggers and thereby modify the list.
+	// Therefore, we create a new temporary list which will receive
+	// triggers triggered during this time.
+	TriggerList* orig = pending;
+	TriggerList tmp;
+	pending = &tmp;
+
+	for ( TriggerList::iterator i = orig->begin(); i != orig->end(); ++i )
+		{
+		Trigger* t = *i;
+		(*i)->Eval();
+		Unref(t);
+		}
+
+	pending = orig;
+	orig->clear();
+
+	std::swap(tmp, *pending);
+	}
+
+void Manager::Queue(Trigger* trigger)
+	{
+	if ( std::find(pending->begin(), pending->end(), trigger) == pending->end() )
+		{
+		Ref(trigger);
+		pending->push_back(trigger);
+		total_triggers++;
+		iosource_mgr->Wakeup(Tag());
+		}
+	}
+
+void Manager::GetStats(Stats* stats)
 	{
 	stats->total = total_triggers;
-	stats->pending = pending ? pending->size() : 0;
+	stats->pending = pending->size();
 	}
+
+}
