@@ -4,6 +4,7 @@
 
 #include "Hash.h"
 #include "Val.h"
+#include "IntrusivePtr.h"
 
 #include "protocol/conn-size/ConnSize.h"
 #include "protocol/icmp/ICMP.h"
@@ -70,9 +71,6 @@ Manager::~Manager()
 	for ( analyzer_map_by_port::const_iterator i = analyzers_by_port_udp.begin(); i != analyzers_by_port_udp.end(); i++ )
 		delete i->second;
 
-	analyzers_by_port_udp.clear();
-	analyzers_by_port_tcp.clear();
-
 	// Clean up expected-connection table.
 	while ( conns_by_timeout.size() )
 		{
@@ -92,26 +90,24 @@ void Manager::InitPreScript()
 
 void Manager::InitPostScript()
 	{
-	auto id = global_scope()->Lookup("Tunnel::vxlan_ports");
+	const auto& id = zeek::detail::global_scope()->Find("Tunnel::vxlan_ports");
 
-	if ( ! (id && id->ID_Val()) )
+	if ( ! (id && id->GetVal()) )
 		reporter->FatalError("Tunnel::vxlan_ports not defined");
 
-	auto table_val = id->ID_Val()->AsTableVal();
-	auto port_list = table_val->ConvertToPureList();
+	auto table_val = id->GetVal()->AsTableVal();
+	auto port_list = table_val->ToPureListVal();
 
 	for ( auto i = 0; i < port_list->Length(); ++i )
-		vxlan_ports.emplace_back(port_list->Index(i)->AsPortVal()->Port());
-
-	Unref(port_list);
+		vxlan_ports.emplace_back(port_list->Idx(i)->AsPortVal()->Port());
 	}
 
 void Manager::DumpDebug()
 	{
 #ifdef DEBUG
 	DBG_LOG(DBG_ANALYZER, "Available analyzers after zeek_init():");
-	list<Component*> all_analyzers = GetComponents();
-	for ( list<Component*>::const_iterator i = all_analyzers.begin(); i != all_analyzers.end(); ++i )
+	std::list<Component*> all_analyzers = GetComponents();
+	for ( std::list<Component*>::const_iterator i = all_analyzers.begin(); i != all_analyzers.end(); ++i )
 		DBG_LOG(DBG_ANALYZER, "    %s (%s)", (*i)->Name().c_str(),
 			IsEnabled((*i)->Tag()) ? "enabled" : "disabled");
 
@@ -120,20 +116,20 @@ void Manager::DumpDebug()
 
 	for ( analyzer_map_by_port::const_iterator i = analyzers_by_port_tcp.begin(); i != analyzers_by_port_tcp.end(); i++ )
 		{
-		string s;
+		std::string s;
 
 		for ( tag_set::const_iterator j = i->second->begin(); j != i->second->end(); j++ )
-			s += string(GetComponentName(*j)) + " ";
+			s += std::string(GetComponentName(*j)) + " ";
 
 		DBG_LOG(DBG_ANALYZER, "    %d/tcp: %s", i->first, s.c_str());
 		}
 
 	for ( analyzer_map_by_port::const_iterator i = analyzers_by_port_udp.begin(); i != analyzers_by_port_udp.end(); i++ )
 		{
-		string s;
+		std::string s;
 
 		for ( tag_set::const_iterator j = i->second->begin(); j != i->second->end(); j++ )
-			s += string(GetComponentName(*j)) + " ";
+			s += std::string(GetComponentName(*j)) + " ";
 
 		DBG_LOG(DBG_ANALYZER, "    %d/udp: %s", i->first, s.c_str());
 		}
@@ -145,7 +141,7 @@ void Manager::Done()
 	{
 	}
 
-bool Manager::EnableAnalyzer(Tag tag)
+bool Manager::EnableAnalyzer(const Tag& tag)
 	{
 	Component* p = Lookup(tag);
 
@@ -158,7 +154,7 @@ bool Manager::EnableAnalyzer(Tag tag)
 	return true;
 	}
 
-bool Manager::EnableAnalyzer(EnumVal* val)
+bool Manager::EnableAnalyzer(zeek::EnumVal* val)
 	{
 	Component* p = Lookup(val);
 
@@ -171,7 +167,7 @@ bool Manager::EnableAnalyzer(EnumVal* val)
 	return true;
 	}
 
-bool Manager::DisableAnalyzer(Tag tag)
+bool Manager::DisableAnalyzer(const Tag& tag)
 	{
 	Component* p = Lookup(tag);
 
@@ -184,7 +180,7 @@ bool Manager::DisableAnalyzer(Tag tag)
 	return true;
 	}
 
-bool Manager::DisableAnalyzer(EnumVal* val)
+bool Manager::DisableAnalyzer(zeek::EnumVal* val)
 	{
 	Component* p = Lookup(val);
 
@@ -201,8 +197,8 @@ void Manager::DisableAllAnalyzers()
 	{
 	DBG_LOG(DBG_ANALYZER, "Disabling all analyzers");
 
-	list<Component*> all_analyzers = GetComponents();
-	for ( list<Component*>::const_iterator i = all_analyzers.begin(); i != all_analyzers.end(); ++i )
+	std::list<Component*> all_analyzers = GetComponents();
+	for ( std::list<Component*>::const_iterator i = all_analyzers.begin(); i != all_analyzers.end(); ++i )
 		(*i)->SetEnabled(false);
 	}
 
@@ -211,7 +207,7 @@ analyzer::Tag Manager::GetAnalyzerTag(const char* name)
 	return GetComponentTag(name);
 	}
 
-bool Manager::IsEnabled(Tag tag)
+bool Manager::IsEnabled(const Tag& tag)
 	{
 	if ( ! tag )
 		return false;
@@ -224,7 +220,7 @@ bool Manager::IsEnabled(Tag tag)
 	return p->Enabled();
 	}
 
-bool Manager::IsEnabled(EnumVal* val)
+bool Manager::IsEnabled(zeek::EnumVal* val)
 	{
 	Component* p = Lookup(val);
 
@@ -235,7 +231,7 @@ bool Manager::IsEnabled(EnumVal* val)
 	}
 
 
-bool Manager::RegisterAnalyzerForPort(EnumVal* val, PortVal* port)
+bool Manager::RegisterAnalyzerForPort(zeek::EnumVal* val, zeek::PortVal* port)
 	{
 	Component* p = Lookup(val);
 
@@ -245,7 +241,7 @@ bool Manager::RegisterAnalyzerForPort(EnumVal* val, PortVal* port)
 	return RegisterAnalyzerForPort(p->Tag(), port->PortType(), port->Port());
 	}
 
-bool Manager::UnregisterAnalyzerForPort(EnumVal* val, PortVal* port)
+bool Manager::UnregisterAnalyzerForPort(zeek::EnumVal* val, zeek::PortVal* port)
 	{
 	Component* p = Lookup(val);
 
@@ -255,7 +251,7 @@ bool Manager::UnregisterAnalyzerForPort(EnumVal* val, PortVal* port)
 	return UnregisterAnalyzerForPort(p->Tag(), port->PortType(), port->Port());
 	}
 
-bool Manager::RegisterAnalyzerForPort(Tag tag, TransportProto proto, uint32_t port)
+bool Manager::RegisterAnalyzerForPort(const Tag& tag, TransportProto proto, uint32_t port)
 	{
 	tag_set* l = LookupPort(proto, port, true);
 
@@ -271,7 +267,7 @@ bool Manager::RegisterAnalyzerForPort(Tag tag, TransportProto proto, uint32_t po
 	return true;
 	}
 
-bool Manager::UnregisterAnalyzerForPort(Tag tag, TransportProto proto, uint32_t port)
+bool Manager::UnregisterAnalyzerForPort(const Tag& tag, TransportProto proto, uint32_t port)
 	{
 	tag_set* l = LookupPort(proto, port, true);
 
@@ -287,24 +283,24 @@ bool Manager::UnregisterAnalyzerForPort(Tag tag, TransportProto proto, uint32_t
 	return true;
 	}
 
-Analyzer* Manager::InstantiateAnalyzer(Tag tag, Connection* conn)
+Analyzer* Manager::InstantiateAnalyzer(const Tag& tag, Connection* conn)
 	{
 	Component* c = Lookup(tag);
 
 	if ( ! c )
 		{
 		reporter->InternalWarning("request to instantiate unknown analyzer");
-		return 0;
+		return nullptr;
 		}
 
 	if ( ! c->Enabled() )
-		return 0;
+		return nullptr;
 
 	if ( ! c->Factory() )
 		{
 		reporter->InternalWarning("analyzer %s cannot be instantiated dynamically",
 					  GetComponentName(tag).c_str());
-		return 0;
+		return nullptr;
 		}
 
 	Analyzer* a = c->Factory()(conn);
@@ -312,7 +308,7 @@ Analyzer* Manager::InstantiateAnalyzer(Tag tag, Connection* conn)
 	if ( ! a )
 		{
 		reporter->InternalWarning("analyzer instantiation failed");
-		return 0;
+		return nullptr;
 		}
 
 	a->SetAnalyzerTag(tag);
@@ -323,12 +319,12 @@ Analyzer* Manager::InstantiateAnalyzer(Tag tag, Connection* conn)
 Analyzer* Manager::InstantiateAnalyzer(const char* name, Connection* conn)
 	{
 	Tag tag = GetComponentTag(name);
-	return tag ? InstantiateAnalyzer(tag, conn) : 0;
+	return tag ? InstantiateAnalyzer(tag, conn) : nullptr;
 	}
 
 Manager::tag_set* Manager::LookupPort(TransportProto proto, uint32_t port, bool add_if_not_found)
 	{
-	analyzer_map_by_port* m = 0;
+	analyzer_map_by_port* m = nullptr;
 
 	switch ( proto ) {
 	case TRANSPORT_TCP:
@@ -341,7 +337,7 @@ Manager::tag_set* Manager::LookupPort(TransportProto proto, uint32_t port, bool
 
 	default:
 		reporter->InternalWarning("unsupported transport protocol in analyzer::Manager::LookupPort");
-		return 0;
+		return nullptr;
 	}
 
 	analyzer_map_by_port::const_iterator i = m->find(port);
@@ -350,25 +346,25 @@ Manager::tag_set* Manager::LookupPort(TransportProto proto, uint32_t port, bool
 		return i->second;
 
 	if ( ! add_if_not_found )
-		return 0;
+		return nullptr;
 
 	tag_set* l = new tag_set;
 	m->insert(std::make_pair(port, l));
 	return l;
 	}
 
-Manager::tag_set* Manager::LookupPort(PortVal* val, bool add_if_not_found)
+Manager::tag_set* Manager::LookupPort(zeek::PortVal* val, bool add_if_not_found)
 	{
 	return LookupPort(val->PortType(), val->Port(), add_if_not_found);
 	}
 
 bool Manager::BuildInitialAnalyzerTree(Connection* conn)
 	{
-	tcp::TCP_Analyzer* tcp = 0;
-	udp::UDP_Analyzer* udp = 0;
-	icmp::ICMP_Analyzer* icmp = 0;
-	TransportLayerAnalyzer* root = 0;
-	pia::PIA* pia = 0;
+	tcp::TCP_Analyzer* tcp = nullptr;
+	udp::UDP_Analyzer* udp = nullptr;
+	icmp::ICMP_Analyzer* icmp = nullptr;
+	TransportLayerAnalyzer* root = nullptr;
+	pia::PIA* pia = nullptr;
 	bool check_port = false;
 
 	switch ( conn->ConnTransport() ) {
@@ -442,16 +438,15 @@ bool Manager::BuildInitialAnalyzerTree(Connection* conn)
 
 		if ( tcp_contents && ! reass )
 			{
-			auto dport = val_mgr->GetPort(ntohs(conn->RespPort()), TRANSPORT_TCP);
-			Val* result;
+			static auto tcp_content_delivery_ports_orig = zeek::id::find_val<zeek::TableVal>("tcp_content_delivery_ports_orig");
+			static auto tcp_content_delivery_ports_resp = zeek::id::find_val<zeek::TableVal>("tcp_content_delivery_ports_resp");
+			const auto& dport = zeek::val_mgr->Port(ntohs(conn->RespPort()), TRANSPORT_TCP);
 
 			if ( ! reass )
-				reass = tcp_content_delivery_ports_orig->Lookup(dport);
+				reass = (bool)tcp_content_delivery_ports_orig->FindOrDefault(dport);
 
 			if ( ! reass )
-				reass = tcp_content_delivery_ports_resp->Lookup(dport);
-
-			Unref(dport);
+				reass = (bool)tcp_content_delivery_ports_resp->FindOrDefault(dport);
 			}
 
 		if ( reass )
@@ -467,8 +462,10 @@ bool Manager::BuildInitialAnalyzerTree(Connection* conn)
 			uint16_t resp_port = ntohs(conn->RespPort());
 			if ( resp_port == 22 || resp_port == 23 || resp_port == 513 )
 				{
-				AddrVal src(conn->OrigAddr());
-				if ( ! stp_skip_src->Lookup(&src) )
+				static auto stp_skip_src = zeek::id::find_val<zeek::TableVal>("stp_skip_src");
+				auto src = zeek::make_intrusive<zeek::AddrVal>(conn->OrigAddr());
+
+				if ( ! stp_skip_src->FindOrDefault(src) )
 					tcp->AddChildAnalyzer(new stepping_stone::SteppingStone_Analyzer(conn), false);
 				}
 			}
@@ -542,7 +539,7 @@ void Manager::ExpireScheduledAnalyzers()
 
 void Manager::ScheduleAnalyzer(const IPAddr& orig, const IPAddr& resp,
 			uint16_t resp_p,
-			TransportProto proto, Tag analyzer,
+			TransportProto proto, const Tag& analyzer,
 			double timeout)
 	{
 	if ( ! network_time )
@@ -576,11 +573,12 @@ void Manager::ScheduleAnalyzer(const IPAddr& orig, const IPAddr& resp,
 		ScheduleAnalyzer(orig, resp, resp_p, proto, tag, timeout);
 	}
 
-void Manager::ScheduleAnalyzer(const IPAddr& orig, const IPAddr& resp, PortVal* resp_p,
-			       Val* analyzer, double timeout)
+void Manager::ScheduleAnalyzer(const IPAddr& orig, const IPAddr& resp, zeek::PortVal* resp_p,
+                               zeek::Val* analyzer, double timeout)
 	{
-	EnumVal* ev = analyzer->AsEnumVal();
-	return ScheduleAnalyzer(orig, resp, resp_p->Port(), resp_p->PortType(), Tag(ev), timeout);
+	zeek::EnumValPtr ev{zeek::NewRef{}, analyzer->AsEnumVal()};
+	return ScheduleAnalyzer(orig, resp, resp_p->Port(), resp_p->PortType(),
+	                        Tag(std::move(ev)), timeout);
 	}
 
 Manager::tag_set Manager::GetScheduled(const Connection* conn)
@@ -629,9 +627,9 @@ bool Manager::ApplyScheduledAnalyzers(Connection* conn, bool init, TransportLaye
 
 		parent->AddChildAnalyzer(analyzer, init);
 
-		EnumVal* tag = it->AsEnumVal();
-		Ref(tag);
-		conn->Event(scheduled_analyzer_applied, 0, tag);
+		if ( scheduled_analyzer_applied )
+			conn->EnqueueEvent(scheduled_analyzer_applied, nullptr,
+			                   conn->ConnVal(), it->AsVal());
 
 		DBG_ANALYZER_ARGS(conn, "activated %s analyzer as scheduled",
 		                  analyzer_mgr->GetComponentName(*it).c_str());
