@@ -1,8 +1,7 @@
 // See the file "COPYING" in the main distribution directory for copyright.
 //
 
-#ifndef ANALYZER_PROTOCOL_CONN_SIZE_CONNSIZE_H
-#define ANALYZER_PROTOCOL_CONN_SIZE_CONNSIZE_H
+#pragma once
 
 #include "analyzer/Analyzer.h"
 #include "NetVar.h"
@@ -18,11 +17,11 @@ public:
 	void Done() override;
 
 	// from Analyzer.h
-	void UpdateConnVal(RecordVal *conn_val) override;
+	void UpdateConnVal(zeek::RecordVal *conn_val) override;
 	void FlipRoles() override;
 
 	void SetByteAndPacketThreshold(uint64_t threshold, bool bytes, bool orig);
-	uint64 GetByteAndPacketThreshold(bool bytes, bool orig);
+	uint64_t GetByteAndPacketThreshold(bool bytes, bool orig);
 
 	void SetDurationThreshold(double duration);
 	double GetDurationThreshold() { return duration_thresh; };
@@ -32,10 +31,10 @@ public:
 
 protected:
 	void DeliverPacket(int len, const u_char* data, bool is_orig,
-					uint64 seq, const IP_Hdr* ip, int caplen) override;
+					   uint64_t seq, const IP_Hdr* ip, int caplen) override;
 	void CheckThresholds(bool is_orig);
 
-	void ThresholdEvent(EventHandlerPtr f, uint64 threshold, bool is_orig);
+	void ThresholdEvent(EventHandlerPtr f, uint64_t threshold, bool is_orig);
 
 	uint64_t orig_bytes;
 	uint64_t resp_bytes;
@@ -51,6 +50,4 @@ protected:
 	double duration_thresh;
 };
 
-} } // namespace analyzer::* 
-
-#endif
+} } // namespace analyzer::*
