@@ -21,7 +21,7 @@ void File_Analyzer::DeliverStream(int len, const u_char* data, bool orig)
 	{
 	tcp::TCP_ApplicationAnalyzer::DeliverStream(len, data, orig);
 
-	int n = min(len, BUFFER_SIZE - buffer_len);
+	int n = std::min(len, BUFFER_SIZE - buffer_len);
 
 	if ( n )
 		{
@@ -40,7 +40,7 @@ void File_Analyzer::DeliverStream(int len, const u_char* data, bool orig)
 		                                orig, file_id_resp);
 	}
 
-void File_Analyzer::Undelivered(uint64 seq, int len, bool orig)
+void File_Analyzer::Undelivered(uint64_t seq, int len, bool orig)
 	{
 	TCP_ApplicationAnalyzer::Undelivered(seq, len, orig);
 
@@ -75,14 +75,15 @@ void File_Analyzer::Identify()
 	RuleMatcher::MIME_Matches matches;
 	file_mgr->DetectMIME(reinterpret_cast<const u_char*>(buffer), buffer_len,
 	                     &matches);
-	string match = matches.empty() ? "<unknown>"
+	std::string match = matches.empty() ? "<unknown>"
 	                               : *(matches.begin()->second.begin());
 
 	if ( file_transferred )
-		ConnectionEventFast(file_transferred, {
-			BuildConnVal(),
-			new StringVal(buffer_len, buffer),
-			new StringVal("<unknown>"),
-			new StringVal(match),
-		});
+		EnqueueConnEvent(
+			file_transferred,
+			ConnVal(),
+			zeek::make_intrusive<zeek::StringVal>(buffer_len, buffer),
+			zeek::make_intrusive<zeek::StringVal>("<unknown>"),
+			zeek::make_intrusive<zeek::StringVal>(match)
+		);
 	}
