@@ -4,24 +4,15 @@
 
 #include "MQTT.h"
 #include "Reporter.h"
+#include "Scope.h"
 #include "mqtt_pac.h"
 
 using namespace analyzer::MQTT;
 
-const ::ID* MQTT_Analyzer::max_payload_size = nullptr;
-
 MQTT_Analyzer::MQTT_Analyzer(Connection* c)
 	: tcp::TCP_ApplicationAnalyzer("MQTT", c)
 	{
 	interp = new binpac::MQTT::MQTT_Conn(this);
-
-	if ( ! max_payload_size )
-		{
-		max_payload_size = global_scope()->Lookup("MQTT::max_payload_size");
-
-		if ( ! max_payload_size )
-			reporter->FatalError("option not defined: 'MQTT::max_payload_size'");
-		}
 	}
 
 MQTT_Analyzer::~MQTT_Analyzer()
@@ -59,7 +50,7 @@ void MQTT_Analyzer::DeliverStream(int len, const u_char* data, bool orig)
 		}
 	}
 
-void MQTT_Analyzer::Undelivered(uint64 seq, int len, bool orig)
+void MQTT_Analyzer::Undelivered(uint64_t seq, int len, bool orig)
 	{
 	tcp::TCP_ApplicationAnalyzer::Undelivered(seq, len, orig);
 	interp->NewGap(orig, len);
