@@ -7,13 +7,13 @@
 namespace plugin {
 namespace Zeek_AsciiReader {
 
-class Plugin : public plugin::Plugin {
+class Plugin : public zeek::plugin::Plugin {
 public:
-	plugin::Configuration Configure()
+	zeek::plugin::Configuration Configure() override
 		{
 		AddComponent(new ::input::Component("Ascii", ::input::reader::Ascii::Instantiate));
 
-		plugin::Configuration config;
+		zeek::plugin::Configuration config;
 		config.name = "Zeek::AsciiReader";
 		config.description = "ASCII input reader";
 		return config;
