@@ -8,13 +8,13 @@
 namespace plugin {
 namespace Zeek_NoneWriter {
 
-class Plugin : public plugin::Plugin {
+class Plugin : public zeek::plugin::Plugin {
 public:
-	plugin::Configuration Configure()
+	zeek::plugin::Configuration Configure() override
 		{
 		AddComponent(new ::logging::Component("None", ::logging::writer::None::Instantiate));
 
-		plugin::Configuration config;
+		zeek::plugin::Configuration config;
 		config.name = "Zeek::NoneWriter";
 		config.description = "None log writer (primarily for debugging)";
 		return config;
