@@ -137,8 +137,8 @@ const char* fmt_conn_id(const IPAddr& src_addr, uint32_t src_port,
 	static char buffer[512];
 
 	snprintf(buffer, sizeof(buffer), "%s:%d > %s:%d",
-			string(src_addr).c_str(), src_port,
-			string(dst_addr).c_str(), dst_port);
+	         std::string(src_addr).c_str(), src_port,
+	         std::string(dst_addr).c_str(), dst_port);
 
 	return buffer;
 	}
